#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2017 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import numpy
from gnuradio import gr

import pmt

from . import csp_header

class swap_crc(gr.basic_block):
    """
    docstring for block swap_crc
    """
    def __init__(self):
        gr.basic_block.__init__(self,
            name="swap_crc",
            in_sig=[],
            out_sig=[])

        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('out'))

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print("[ERROR] Received invalid message type. Expected u8vector")
            return
        packet = bytes(pmt.u8vector_elements(msg))
        try:
            header = csp_header.CSP(packet[:4])
        except ValueError as e:
            return
        if not header.crc:
            self.message_port_pub(pmt.intern('out'), msg_pmt)
        else:
            if len(packet) < 8: # bytes CSP header, 4 bytes CRC-32C
                # malformed
                return
            crc = packet[-4:]
            packet = packet[:-4] + crc[::-1]
            msg_pmt = pmt.cons(pmt.PMT_NIL, pmt.init_u8vector(len(packet), packet))
            self.message_port_pub(pmt.intern('out'), msg_pmt)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Athanasios Theocharis <athatheoc@gmail.com>
# This was made under ESA Summer of Code in Space 2019
# by Athanasios Theocharis, mentored by Daniel Estevez
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import numpy
from gnuradio import gr
import pmt
from . import space_packet
import array

class space_packet_primaryheader_adder(gr.basic_block):
    """
    docstring for block space_packet_primaryheader_adder
    """
    def __init__(self, packet_type, secondary_header_flag, AP_ID, count_or_name, packet_sequence_name):
        gr.basic_block.__init__(self,
            name="space_packet_primaryheader_adder",
            in_sig=[],
            out_sig=[])

        ##################################################
        # Parameters
        ##################################################
        self.ccsds_version = 0
        self.packet_type = packet_type
        self.secondary_header_flag = secondary_header_flag
        self.AP_ID = AP_ID
        self.sequence_flags = 3
        self.packet_sequence_count = 0
        self.packet_sequence_name = packet_sequence_name
        self.count_or_name = count_or_name
        self.data_length = 0

        ##################################################
        # Blocks
        ##################################################
        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('out'))

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print("[ERROR] Received invalid message type. Expected u8vector")
            return
        packet = pmt.u8vector_elements(msg)
        self.data_length = len(packet)
        self.packet_sequence_count += 1
        count_or_name = self.packet_sequence_count if self.packet_type == 0 or self.count_or_name == 0 else self.packet_sequence_name

        finalHeader = array.array('B', space_packet.PrimaryHeader.build(dict(ccsds_version = self.ccsds_version,
                                                                packet_type = self.packet_type,
                                                                secondary_header_flag = self.secondary_header_flag,
                                                                AP_ID = self.AP_ID,
                                                                sequence_flags = self.sequence_flags,
                                                                packet_sequence_count_or_name = count_or_name,
                                                                data_length = self.data_length))).tolist()

        finalPacket = numpy.append(finalHeader, packet)
        finalPacket = array.array('B', finalPacket[:])
        finalPacket = pmt.cons(pmt.PMT_NIL, pmt.init_u8vector(len(finalPacket), finalPacket))
        self.message_port_pub(pmt.intern('out'), finalPacket)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2018-2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import numpy
from gnuradio import gr
import pmt
import array

class aausat4_remove_fsm(gr.basic_block):
    """
    docstring for block aausat4_remove_fsm
    """
    def __init__(self):
        gr.basic_block.__init__(self,
            name="aausat4_remove_fsm",
            in_sig=[],
            out_sig=[])

        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('short'))
        self.message_port_register_out(pmt.intern('long'))

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_f32vector(msg):
            print("[ERROR] Received invalid message type. Expected f32vector")
            return
        packet_short = pmt.f32vector_elements(msg)[8:8+1020]
        packet_long = pmt.f32vector_elements(msg)[8:8+1996]
        self.message_port_pub(pmt.intern('short'),
                              pmt.cons(pmt.PMT_NIL, pmt.init_f32vector(len(packet_short), packet_short)))
        self.message_port_pub(pmt.intern('long'),
                              pmt.cons(pmt.PMT_NIL, pmt.init_f32vector(len(packet_long), packet_long)))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from construct import *
from ..adapters import LinearAdapter
from .csp import CSPHeader

EPS = Struct(
    'boot_count' / Int16ub,
    'uptime' / Int32ub,
    'rt_clock' / Int32ub,
    'ping_status' / Int8ub,
    'subsystem_status' / Int16ub,
    'battery_voltage' / LinearAdapter(1/40.0, Int8ub),
    'cell_diff' / LinearAdapter(1/4.0, Int8sb),
    'battery_current' / LinearAdapter(1/10.0, Int8sb),
    'solar_power' / LinearAdapter(1/20.0, Int8ub),
    'temp' / Int8sb,
    'pa_temp' / Int8sb,
    'main_voltage' / Int8sb
    )

COM = Struct(
    'boot_count' / Int16ub,
    'packets_received' / Int16ub,
    'packets_sent' / Int16ub,
    'latest_rssi' / Int16sb,
    'latest_bit_correction' / Int8ub,
    'latest_byte_correction' / Int8ub
    )

# Reverse-engineered

ADCS1 = Struct(
    'bdot' / Int16sb[3],
    'state' / Int8ub
    )

ADCS2 = Struct(
    'gyro' / Int16sb[3]
    )

AIS = Struct(
    'boot_count' / Int16ub,
    Padding(4), # unknown data
    'unique_mssi' / Int16ub,
    Padding(12) # unknown data
    )

Valid = BitStruct(
    Padding(2),
    'ais2' / Flag,
    'ais1' / Flag,
    'adcs2' / Flag,
    'adcs1' / Flag,
    'com' / Flag,
    'eps' / Flag
    )

aausat4 = Struct(
    'frame_length' / Int16ub,
    'csp_header' / CSPHeader,
    'valid' / Valid,
    'eps' / EPS,
    'com' / COM,
    'adcs1' / ADCS1,
    'adcs2' / ADCS2,
    'ais1' / AIS,
    'ais2' / AIS
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2018-2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
# 

from construct import *
from ..ccsds import space_packet as ccsds_space_packet
from .. import ecss_pus

from ..adapters import LinearAdapter

from .csp import CSPHeader

TMHeader = BitStruct(
    'version' / BitsInteger(2),
    'sc_id' / BitsInteger(10),
    'virtual_channel_id' / BitsInteger(4),
    'virtual_channel_frame_counter' / BitsInteger(8),
    'first_header_pointer' / BitsInteger(11),
    'empty_frame' / BitsInteger(1),
    'ocf_presence' / BitsInteger(1),
    'sequence_flags' / BitsInteger(2),
    'fixed_length_frame' / BitsInteger(1))

TMTail = Struct(
    'packet_errors' / Int16ub,
    'frame_errors' / Int16ub,
    'frame_error_control' / Int16ub)

SPTail = Struct(
    'PEC' / Int16ub)

TimeField = Struct(
    'day' / Int16ub,
    'milliseconds_of_day' / Int32ub)

OBC = Struct(
    'boot_cause' / Int32ub,
    'boot_count' / Int16ub,
    'clock' / Int32ub,
    'curr_flash' / Int16ub,
    'fs_mounted' / Int8ub,
    'ram_image' / Int8sb,
    'temp' / LinearAdapter(10, Int16sb)[2],
    'ticks' / Int32ub,
    'mag' / Float32b[3],
    'memfree' / Int32ub,
    'bufferfree' / Int32ub,
    'uptime' / Int32ub,
    'gyro' / Float32b[3],
    'gyro_temp' / Float32b,
    'flash_total' / Int64ub,
    'flash_used' / Int64ub,
    'flash_free' / Int64ub,
    'gpio_test' / Int8ub,
    'gpio_sw' / Int8ub,
    'gpio_pwr' / Int8ub,
    'om_state' / Int8ub,
    'om_sw_version' / Bytes(32),
    'op_tr_conn' / Int8ub,
    'op_tr_conn_active' / Int8ub)

EPS = Struct(
    'output_off_delta' / Int16ub[8],
    'output_on_delta' / Int16ub[8],
    'wdt_csp_pings_left' / Int8ub[2],
    'bootcause' / Int8ub,
    'cursun' / Int16ub,
    'curin' / Int16ub[3],
    'curout' / Int16ub[6],
    'cursys' / Int16ub,
    'temp' / Int16ub[6],
    'battmode' / Int8ub,
    'pptmode' / Int8ub,
    'counter_boot' / Int32ub,
    'latchup' / Int16ub[6],
    'counter_wdt_csp' / Int32ub[2],
    'counter_wdt_gnd' / Int32ub,
    'counter_wdt_i2c' / Int32ub,
    'output' / Int8ub[8],
    'wdt_gnd_time_left' / Int32ub,
    'wdt_i2c_time_left' / Int32ub,
    'vbatt' / Int16ub,
    'vboost' / Int16ub[3],
    'wdtcspc' / Int8ub[2])

TTC = Struct(
    'temp_brd' / LinearAdapter(10, Int16sb),
    'last_rferr' / Int16sb,
    'last_rssi' / Int16sb,
    'tot_rx_bytes' / Int32ub,
    'rx_bytes' / Int32ub,
    'tot_rx_count' / Int32ub,
    'rx_count' / Int32ub,
    'tot_tx_bytes' / Int32ub,
    'tx_bytes' / Int32ub,
    'tot_tx_count' / Int32ub,
    'tx_count' / Int32ub,
    'temp_pa' / LinearAdapter(10, Int16sb),
    'boot_cause' / Int32ub,
    'bgnd_rssi' / Int16sb,
    'active_conf' / Int8ub,
    'boot_count' / Int16ub,
    'last_contact' / Int32ub,
    'tx_duty' / Int8ub)

GSSBEntry = Struct(
    'reboots' / Int8ub,
    'current_state' / Int8ub,
    'antenna_state' / Int8ub,
    'attempts_total' / Int16ub)

GSSB =  GSSBEntry[4]

TTCGSSB = Struct(
    'gssb' / GSSB,
    'ttc' / TTC)

AOCS = Struct(
    'extmag_valid' / Int8ub,
    'extmag' / Float32b[3],
    'gps_pos_dev' / Float32b[3],
    'gps_pos' / Float32b[3],
    'gps_valid' / Int8ub,
    'gyro_valid' / Int8ub,
    'gyro' / Float32b[3],
    'mag' / Float32b[3],
    'mag_valid' / Int8ub,
    'status_run' / Int8sb,
    'acs_mode' / Int8sb,
    'ads_mode' / Int8sb,
    'ephem_mode' / Int8sb,
    'bdot_detumb' / Int8ub,
    'boot_cause' / Int32ub,
    'boot_count' / Int16ub,
    'cur_gssb' / Int16ub[2],
    'cur_pwm' / Int16ub,
    'cur_gps' / Int16ub,
    'cur_wde' / Int16ub)

Temps = Struct(
    'aocs_suns' / Float32b[5],
    'not_used' / Float32b,
    'aocs_extmag' / Float32b,
    'aocs_fss' / Float32b[5],
    'not_used2' / Float32b[3],
    'aocs_gyro' / Float32b,
    'aocs' / LinearAdapter(10, Int16sb)[2],
    'eps' / Int16sb[6],
    'obc' / LinearAdapter(10, Int16sb)[2],
    'obc_gyro' / Float32b,
    'ttc_brd' / LinearAdapter(10, Int16sb),
    'ttc_pa' / LinearAdapter(10, Int16sb))

payloads = {1 : OBC, 2 : EPS, 3 : TTCGSSB, 4 : AOCS, 5: Temps}

UserData = Struct(
    'id' / Int16ub,
    'data' / Switch(this.id, payloads, default = Bytes(this._.space_packet_header.data_length-16))
    )

lume = Struct(
    'csp_header' / CSPHeader,
    'tm_header' / TMHeader,
    'space_packet_header' / ccsds_space_packet.PrimaryHeader,
    'pus_header' / ecss_pus.TMSecondaryHeader,
    'pus_time_field' / TimeField,
    'payload' / UserData,
    'space_packet_tail' / SPTail,
    'tm_tail' / TMTail)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Athanasios Theocharis <athatheoc@gmail.com>
# This was made under ESA Summer of Code in Space 2019
# by Athanasios Theocharis, mentored by Daniel Estevez
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
# 

import numpy
from gnuradio import gr
import pmt
from . import space_packet

class space_packet_parser(gr.basic_block):
    """

    """

    def __init__(self, time_header, time_format, pfield, id_time):
        gr.basic_block.__init__(self,
                                name="Space Packet Parser",
                                in_sig=[],
                                out_sig=[])
        self.time_header = time_header
        self.time_format = time_format if time_header == 0 else 0
        self.pfield = pfield if time_header == 0 else 0
        self.basic_time_num_octets_cuc = id_time.basic_time_num_octets_cuc if time_header == 0 else 0
        self.fractional_time_num_octets_cuc = id_time.fractional_time_num_octets_cuc if time_header == 0 else 0
        self.additional_octets_basic_time_cuc = id_time.additional_basic_time_num_octets_cuc if time_header == 0 else 0
        self.additional_octets_fractional_time_cuc = id_time.additional_fractional_time_num_octets_cuc if time_header == 0 else 0
        self.length_of_day_cds = id_time.len_of_day if time_header == 0 else 0
        self.length_of_submillisecond_cds = id_time.len_of_submilsecs if time_header == 0 else 0
        self.calendar_variation_ccs = id_time.calendar_variation if time_header == 0 else 0
        self.number_of_subsecond_ccs = id_time.num_of_subsecs if time_header == 0 else 0
        self.add_z_terminator = id_time.add_z if time_header == 0 else 0
        self.ascii_dec_num = id_time.ascii_dec if time_header == 0 else 0


        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print("[ERROR] Received invalid message type. Expected u8vector")
            return
        packet = bytearray(pmt.u8vector_elements(msg))
        if self.pfield == 1:
            packet_formats = [space_packet.FullPacketCUCWithPField, space_packet.FullPacketCDSWithPField,
                              space_packet.FullPacketCCSWithPField]
        else:
            packet_formats = [space_packet.FullPacketCUCNoPField, space_packet.FullPacketCDSNoPField,
                              space_packet.FullPacketCCSNoPField]

        packet_formats.extend([space_packet.FullPacketASCIIA, space_packet.FullPacketASCIIB,
                              space_packet.FullPacketNoTimeStamp])
        try:
            if self.time_header == 0:
                try:
                    packet_format = packet_formats[self.time_format]
                except IndexError:
                    print("Time Format Unknown")
                    return
            else:
                packet_format = packet_formats[5]

            if self.pfield == 0 and self.time_format <= 2 and self.time_header == 0:
                if self.time_format == 0:
                    data = packet_format.parse(packet[:], num_of_basic_time_units = 1+ self.basic_time_num_octets_cuc + self.additional_octets_basic_time_cuc, num_of_fractional_time_units = self.fractional_time_num_octets_cuc + self.additional_octets_fractional_time_cuc)
                elif self.time_format == 1:
                    data = packet_format.parse(packet[:], length_of_day_segment = self.length_of_day_cds, length_of_submillisecond_segment = self.length_of_submillisecond_cds)
                else:
                    data = packet_format.parse(packet[:], calendar_variation_flag = self.calendar_variation_ccs, resolution = self.number_of_subsecond_ccs)
            else:
                if self.time_header == 0 and self.time_format > 2:
                    if self.ascii_dec_num < 0 or self.ascii_dec_num > 6:
                        print()
                        "Decimals of ASCII in Space Packet Parser block should be between 0 and 6. The number was automatically set to 1."
                        self.ascii_dec_num = 1
                    data = packet_format.parse(packet[:], number_of_decimals=self.ascii_dec_num, add_Z=self.add_z_terminator)
                else:
                    data = packet_format.parse(packet[:])
        except:
            print("Could not decode space packet")
            return
        print(data)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, blocks, digital
from .ccsds_rs_deframer import _syncword
from ...hier.ccsds_viterbi import ccsds_viterbi
from ...utils.options_block import options_block
from ... import lilacsat1_demux

class lilacsat_1_deframer(gr.hier_block2, options_block):
    """
    Hierarchical block to deframe LilacSat-1 frames

    The input is a float stream of soft symbols. The output are PDUs
    with chunks of a KISS stream. The additional output is codec2 data.

    Args:
        syncword_threshold: number of bit errors allowed in syncword (int)
        options: Options from argparse
    """
    def __init__(self, syncword_threshold = None, options = None):
        gr.hier_block2.__init__(self, "lilacsat_1_deframer",
            gr.io_signature(1, 1, gr.sizeof_float),
            gr.io_signature(0, 0, 0))
        options_block.__init__(self, options)

        self.message_port_register_hier_out('out')
        self.message_port_register_hier_out('codec2')

        if syncword_threshold is None:
            syncword_threshold = self.options.syncword_threshold

        self.delay1 =  blocks.delay(gr.sizeof_float, 1)
        self.viterbi0 = ccsds_viterbi()
        self.viterbi1 = ccsds_viterbi()
        self.differential0 = digital.diff_decoder_bb(2)
        self.differential1 = digital.diff_decoder_bb(2)
        self.tag0 = digital.correlate_access_code_tag_bb(_syncword, syncword_threshold, 'syncword')
        self.tag1 = digital.correlate_access_code_tag_bb(_syncword, syncword_threshold, 'syncword')
        self.scrambler0 = digital.additive_scrambler_bb(0xA9, 0xFF, 7, count=0, bits_per_byte=1, reset_tag_key='syncword')
        self.scrambler1 = digital.additive_scrambler_bb(0xA9, 0xFF, 7, count=0, bits_per_byte=1, reset_tag_key='syncword')
        self.demux0 = lilacsat1_demux('syncword')
        self.demux1 = lilacsat1_demux('syncword')

        self.connect(self, self.viterbi0, self.differential0, self.tag0, self.scrambler0, self.demux0)
        self.connect(self, self.delay1, self.viterbi1, self.differential1, self.tag1, self.scrambler1, self.demux1)
        self.msg_connect((self.demux0, 'frame'), (self, 'out'))
        self.msg_connect((self.demux1, 'frame'), (self, 'out'))
        self.msg_connect((self.demux0, 'codec2'), (self, 'codec2'))
        self.msg_connect((self.demux1, 'codec2'), (self, 'codec2'))

    _default_sync_threshold = 4

    @classmethod
    def add_options(cls, parser):
        """
        Adds LilacSat-1 deframer specific options to the argparse parser
        """
        parser.add_argument('--syncword_threshold', type = int, default = cls._default_sync_threshold, help = 'Syncword bit errors [default=%(default)r]')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

'''
gr-satellites hierarchical flowgraphs

These are Python files compiled from hierarchical flowgraphs
'''

from .ccsds_descrambler import ccsds_descrambler
from .ccsds_viterbi import ccsds_viterbi
from .pn9_scrambler import pn9_scrambler
from .rms_agc import rms_agc
from .rms_agc_f import rms_agc_f
from .si4463_scrambler import si4463_scrambler
from .sync_to_pdu import sync_to_pdu
from .sync_to_pdu_packed import sync_to_pdu_packed
from .sync_to_pdu_soft import sync_to_pdu_soft




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2016 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
# 

import numpy
from gnuradio import gr
import pmt
import struct

class beesat_classifier(gr.basic_block):
    """
    docstring for block beesat_classifier
    """
    def __init__(self):
        gr.basic_block.__init__(self,
            name="beesat_classifier",
            in_sig=[],
            out_sig=[])

        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('BEESAT-1'))
        self.message_port_register_out(pmt.intern('BEESAT-2'))
        self.message_port_register_out(pmt.intern('BEESAT-4'))
        self.message_port_register_out(pmt.intern('BEESAT-9'))
        self.message_port_register_out(pmt.intern('TECHNOSAT'))

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print("[ERROR] Received invalid message type. Expected u8vector")
            return
        packet = bytes(pmt.u8vector_elements(msg))
        satellite = None
        if len(packet) < 2 + 6 + 2:  # 2byte header, 6byte callsign, 2byte callsign crc
            return
        callsign = struct.unpack("6s", packet[2:8])[0]
        if callsign == 'DP0BEE':
            satellite = 'BEESAT-1'
        elif callsign == 'DP0BEF':
            satellite = 'BEESAT-2'
        elif callsign == 'DP0BEH':
            satellite = 'BEESAT-4'
        elif callsign == 'DP0BEM':
            satellite = 'BEESAT-9'
        elif callsign == 'DP0TBA':
            satellite = 'TECHNOSAT'

        if satellite:
            self.message_port_pub(pmt.intern(satellite), msg_pmt)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, digital
from ... import reflect_bytes, swiatowid_packet_crop, swiatowid_packet_split, decode_rs
from ...hier.sync_to_pdu_packed import sync_to_pdu_packed
from ...utils.options_block import options_block

_syncword = '01011011010110111101110111011101'

class swiatowid_deframer(gr.hier_block2, options_block):
    """
    Hierarchical block to deframe the Swiatowid custom image protocol

    The input is a float stream of soft symbols. The output are PDUs
    with image frames.

    Args:
        syncword_threshold: number of bit errors allowed in syncword (int)
        options: Options from argparse
    """
    def __init__(self, syncword_threshold = None, options = None):
        gr.hier_block2.__init__(self, "swiatowid_deframer",
            gr.io_signature(1, 1, gr.sizeof_float),
            gr.io_signature(0, 0, 0))
        options_block.__init__(self, options)
        
        self.message_port_register_hier_out('out')

        if syncword_threshold is None:
            syncword_threshold = self.options.syncword_threshold

        self.slicer = digital.binary_slicer_fb()
        self.deframer = sync_to_pdu_packed(packlen = 8200,\
                                    sync = _syncword,\
                                    threshold = syncword_threshold)
        self.reflect = reflect_bytes()
        self.crop = swiatowid_packet_crop()
        self.split = swiatowid_packet_split()
        self.rs = decode_rs(8, 0x11d, 0, 1, 10, 1)
        
        self.connect(self, self.slicer, self.deframer)
        self.msg_connect((self.deframer, 'out'), (self.reflect, 'in'))
        self.msg_connect((self.reflect, 'out'), (self.crop, 'in'))
        self.msg_connect((self.crop, 'out'), (self.split, 'in'))
        self.msg_connect((self.split, 'out'), (self.rs, 'in'))
        self.msg_connect((self.rs, 'out'), (self, 'out'))

    _default_sync_threshold = 0
        
    @classmethod
    def add_options(cls, parser):
        """
        Adds Swiatowid deframer specific options to the argparse parser
        """
        parser.add_argument('--syncword_threshold', type = int, default = cls._default_sync_threshold, help = 'Syncword bit errors [default=%(default)r]')
        parser.add_argument('--verbose_rs', action = 'store_true', help = 'Verbose RS decoder')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

'''
gr-satellites telemetry definitions

These are telemetry defintions. Most of them are just a construct
object, but in more complex cases, another class supporting the
.parse method and relying on construct is used.
'''

import construct
construct.setGlobalPrintFullStrings(True)

from .aausat4 import aausat4
from .amicalsat import amicalsat
from .au03 import au03
from .ax25 import ax25
from .by02 import by02
from .by70_1 import by70_1
from .csp import csp
from .csp import csp_swapped
from .dstar_one import dstar_one
from .eseo import eseo
from .floripasat import floripasat
from .fossasat_1b import fossasat_1b
from .fossasat_2 import fossasat_2
from .funcube import funcube
from .gomx_1 import gomx_1
from .gomx_3 import gomx_3
from .kr01 import kr01
from .lume import lume
from .mysat1 import mysat1
from .picsat import picsat
from .qo100 import qo100
from .quetzal1 import quetzal1
from .sat_1kuns_pf import sat_1kuns_pf
from .sat_3cat_1 import sat_3cat_1
from .sat_3cat_2 import sat_3cat_2
from .smogp import smogp
from .smogp import smogp_signalling
from .snet import snet
from .suomi100 import suomi100
from .by70_1 import taurus1
from .trisat import trisat
from .upmsat_2 import upmsat_2
from .vzlusat_2 import vzlusat_2



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2018 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import numpy
from gnuradio import gr
import pmt

from .hdlc_deframer import fcs_ok

class check_astrocast_crc(gr.basic_block):
    """
    docstring for block check_astrocast_crc
    """
    def __init__(self, verbose):
        gr.basic_block.__init__(self,
            name="check_astrocast_crc",
            in_sig=[],
            out_sig=[])

        self.verbose = verbose
        
        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('ok'))
        self.message_port_register_out(pmt.intern('fail'))

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print("[ERROR] Received invalid message type. Expected u8vector")
            return
        packet = pmt.u8vector_elements(msg)[1:] # drop initial 0x7e

        # find final 0x7e
        try:
            idx = packet.index(0x7e)
        except ValueError:
            return
        
        packet_out = packet[:idx-2]
        msg_out = pmt.cons(pmt.car(msg_pmt), pmt.init_u8vector(len(packet_out), packet_out))
        if fcs_ok(packet[:idx]):
            if self.verbose:
                print("CRC OK")
            self.message_port_pub(pmt.intern('ok'), msg_out)
        else:
            if self.verbose:
                print("CRC failed")
            self.message_port_pub(pmt.intern('fail'), msg_out)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
# 

from construct import *
from ..adapters import AffineAdapter, LinearAdapter

Id = Enum(Int8ub,\
              beacon_ngham_obdh_data = 0x00,\
              beacon_ngham_eps_data = 0x01,\
              beacon_ngham_ttc_data = 0x02,\
              beacon_ax25_obdh_data = 0x03,\
              beacon_ax25_eps_data = 0x04,\
              beacon_ax25_ttc_data = 0x05,\
              downlink_telemetry = 0x10,\
              downlink_ping_answer = 0x11,\
              downlink_data_request_answer = 0x12,\
              downlink_hibernation_feedback = 0x13,\
              downlink_charge_reset_feedback = 0x14,\
              downlink_message_broadcast = 0x15,\
              downlink_payload_x_status = 0x16,\
              downlink_rush_status = 0x17,\
              uplink_ping_request = 0x20,\
              uplink_data_request = 0x21,\
              uplink_enter_hibernation = 0x22,\
              uplink_leave_hibernation = 0x23,\
              uplink_charge_reset = 0x24,\
              uplink_broadcast_message = 0x25,\
              uplink_payload_x_status_request = 0x26,\
              uplink_payload_x_status_swap = 0x27,\
              uplink_payload_x_data_upload = 0x28,\
              uplink_rush_enable = 0x29\
              )

BatteryVoltage = LinearAdapter(32/4.883e-3, Int16ub)
BatteryTemperature = LinearAdapter(32/0.125, Int24ub)
BatteryCharge = LinearAdapter(1/6.25e-4, Int16ub)
SolarPanelCurrent = LinearAdapter(1/((2.5/4095)*(1/(0.05*0.025*3300))), Int16ub)
SolarPanelVoltage = AffineAdapter(4095/2.5, -93.1/100, Int16ub)
ImuAccel = LinearAdapter(32768.0/16.0, Int16sb)
ImuGyro = LinearAdapter(32768.0/250, Int16sb)

OBDHStatus = BitStruct(
    Padding(3),
    'antenna' / Flag,
    'imu' / Flag,
    'sd_card' / Flag,
    'rush' / Flag,
    'eps' / Flag
    )

SystemTime = Struct(
    'seconds' / Int8ub,
    'minutes' / Int24ub,
    )

EPS = Struct(
    'battery_voltage' / BatteryVoltage[2],
    'battery_temperature' / BatteryTemperature[2],
    'battery_charge' / BatteryCharge,
    'solar_panel_current' / SolarPanelCurrent[6],
    'solar_panel_voltage' / SolarPanelVoltage[3],
    'energy_level' / Int8ub,
    )

OBDH = Struct(
    'eps' / EPS,
    'status' / OBDHStatus,
    'imu_accelerometer' / ImuAccel[3],
    'imu_gyroscope' / ImuGyro[3],
    'system_time' / SystemTime,
    'odbh_resets' / Int8ub
    )

floripasat = Struct(
    'ngham_padding' / Int8ub,
    'id' / Id,
    'callsign' / Bytes(7),
    'payload' / Switch(this.id, {
        'beacon_ngham_obdh_data' : OBDH,
        'beacon_ngham_eps_data' : EPS,
        'beacon_ax25_obdh_data' : OBDH,
        'beacon_ax25_eps_data' : EPS,
        }, default = GreedyBytes)
    )



#!D:\bld\gnuradio-satellites_1615173903224\_h_env\python.exe
# -*- coding: utf-8 -*-

# Copyright 2019-2020 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import satellites.core
from satellites.utils.config import open_config
import satellites.components.datasources as datasources
from satellites.satyaml import yamlfiles
from gnuradio import gr, blocks, audio

import signal
import argparse
import sys

version = f"""gr_satellites {satellites.__version__}
Copyright (C) 2020 Daniel Estevez
License GPLv3+: GNU GPL version 3 or later <https://gnu.org/licenses/gpl.html>.
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law."""

class ListSatellites(argparse.Action):
    """argparse Action to list supported satellites and exit"""
    def __call__(self, parser, namespace, values, option_string = None):
        print(f'Satellites supported by gr_satellites {satellites.__version__}:')
        ymls = yamlfiles.load_all_yaml()
        for yml in sorted(ymls, key = lambda x: x['name'].casefold()):
            self.__print(yml)
        sys.exit(0)
    def __print(self, yml):
        print(f"* {yml['name']} (NORAD {yml['norad']})")
        for name, data in yml['transmitters'].items():
            print(f"    {name} {data['frequency']*1e-6:.3f} MHz {data['modulation']} {data['framing']}")

def argument_parser():
    description = 'gr-satellites - GNU Radio decoders for Amateur satellites'
    info = 'The satellite parameter can be specified using name, NORAD ID or path to YAML file'
    p = argparse.ArgumentParser(description = description, conflict_handler = 'resolve', prog = f'gr_satellites satellite', epilog = info, formatter_class = argparse.RawTextHelpFormatter)

    p.add_argument('--version', action = 'version', version = version)
    p.add_argument('--list_satellites', action = ListSatellites, nargs = 0, help = 'list supported satellites and exit')
    p.add_argument('--ignore_unknown_args', action = 'store_true', help = 'Treat unknown arguments as warning')

    p_input = p.add_argument_group('input')

    p_sources = p_input.add_mutually_exclusive_group(required = True)
    p_sources.add_argument('--wavfile', help = 'WAV/OGG/FLAC input file (using libsndfile)')
    p_sources.add_argument('--rawfile', help = 'RAW input file (float32 or complex64)')
    p_sources.add_argument('--rawint16', help = 'RAW input file (int16)')
    p_sources.add_argument('--audio', const = '', nargs = '?', metavar = 'DEVICE',  help = 'Soundcard device input')
    p_sources.add_argument('--udp', action = 'store_true', help = 'Use UDP input')
    p_sources.add_argument('--kiss_in', help = 'KISS input file')

    p_input.add_argument('--samp_rate', type = float, help = 'Sample rate (Hz)')
    p_input.add_argument('--udp_ip', default = '::', help = 'UDP input listen IP [default=%(default)r]')
    p_input.add_argument('--udp_port', default = '7355', type = int, help = 'UDP input listen port [default=%(default)r]')
    p_input.add_argument('--iq', action = 'store_true', help = 'Use IQ input')
    p_input.add_argument('--udp_raw', action = 'store_true', help = 'Use RAW UDP input (float32 or complex64)')
    p_input.add_argument('--input_gain', type = float, default = 1, help = 'Input gain (can be negative to invert signal) [default=%(default)r]')
    p_input.add_argument('--start_time', type = str, default = '', help = 'Recording start timestamp')
    p_input.add_argument('--throttle', action = 'store_true', help = 'Throttle recording input to 1x speed')
    
    p_output = p.add_argument_group('output')
    p_output.add_argument('--kiss_out', help = 'KISS output file')
    p_output.add_argument('--kiss_append', action = 'store_true', help = 'Append to KISS output file')
    p_output.add_argument('--kiss_server', const = '8100', nargs = '?', metavar = 'PORT', help = 'Enable KISS server [default port=8100]')
    p_output.add_argument('--kiss_server_address', default = '127.0.0.1', help = 'KISS server bind address [default=%(default)r]')
    p_output.add_argument('--zmq_pub', const = 'tcp://127.0.0.1:5555', nargs = '?', metavar = 'ADDRESS', help = 'Enable ZMQ PUB socket [default address=tcp://127.0.0.1:5555]')
    p_output.add_argument('--hexdump', action = 'store_true', help = 'Hexdump instead of telemetry parse')
    p_output.add_argument('--dump_path', help = 'Path to dump internal signals')
    return p

def check_options(options, parser):
    if options.kiss_in is None and options.wavfile is None and options.samp_rate is None:
        print('Need to specify --samp_rate unless --wavfile or --kiss_in is used', file = sys.stderr)
        parser.print_usage(file = sys.stderr)
        sys.exit(1)

def parse_satellite(satellite):
    if satellite.lower().endswith('.yml'):
        return {'file' : satellite}
    elif satellite.isnumeric():
        return {'norad' : int(satellite)}
    else:
        return {'name' : satellite}

class gr_satellites_top_block(gr.top_block):
    def __init__(self, parser):
        gr.top_block.__init__(self, 'gr-satellites top block')
        sat = parse_satellite(sys.argv[1])
        try:
            satellites.core.gr_satellites_flowgraph.add_options(parser, **sat)
        except ValueError as e:
            if e.args[0] == 'satellite not found':
                print(f'Satellite {sys.argv[1]} not found')
                exit(1)
            else:
                raise e
        except FileNotFoundError as e:
            print('Could not open SatYAML file:')
            print(e)
            exit(1)
        if '--ignore_unknown_args' in sys.argv[2:]:
            options, unknown = parser.parse_known_args(sys.argv[2:])
            if len(unknown) > 0:
                print('Warning: unknown arguments passed {}'.format(unknown), file = sys.stderr)
        else:
            options = parser.parse_args(sys.argv[2:])
        if options.list_satellites:
            list_satellites()
            sys.exit(0)
        check_options(options, parser)
        
        self.options = options

        pdu_in = options.kiss_in is not None

        self.config = open_config()

        self.setup_input()

        # set appropriate sample rate
        if options.samp_rate:
            samp_rate = options.samp_rate
        elif options.wavfile:
            samp_rate = self.wavfile_source.sample_rate()
        else:
            raise ValueError('sample rate not set')
            
        self.flowgraph = satellites.core.gr_satellites_flowgraph(samp_rate = samp_rate,
                                                                 iq = self.options.iq,
                                                                 pdu_in = pdu_in,
                                                                 options = options,
                                                                 config = self.config,
                                                                 dump_path = options.dump_path,
                                                                 **sat)

        if pdu_in:
            self.msg_connect((self.input, 'out'), (self.flowgraph, 'in'))
        else:
            gain = self.options.input_gain
            self.gain = blocks.multiply_const_cc(gain) if self.options.iq else\
              blocks.multiply_const_ff(gain)
            if self.options.throttle:
                size = gr.sizeof_gr_complex if self.options.iq else gr.sizeof_float
                self.throttle = blocks.throttle(size, self.options.samp_rate, True)
                self.connect(self.input, self.throttle, self.gain, self.flowgraph)
            else:
                self.connect(self.input, self.gain, self.flowgraph)

    def setup_input(self):
        if self.options.wavfile is not None:
            return self.setup_wavfile_input()
        elif self.options.rawfile is not None:
            return self.setup_rawfile_input()
        elif self.options.rawint16 is not None:
            return self.setup_rawint16_input()
        elif self.options.audio is not None:
            return self.setup_audio_input()
        elif self.options.udp is True:
            return self.setup_udp_input()
        elif self.options.kiss_in is not None:
            return self.setup_kiss_input()
        else:
            raise Exception('No input source set for flowgraph')

    def setup_wavfile_input(self):
        self.wavfile_source = blocks.wavfile_source(self.options.wavfile, False)
        if self.options.iq:
            self.float_to_complex = blocks.float_to_complex(1)
            self.connect((self.wavfile_source, 0), (self.float_to_complex, 0))
            self.connect((self.wavfile_source, 1), (self.float_to_complex, 1))
            self.input = self.float_to_complex
        else:
            self.input = self.wavfile_source

    def setup_rawfile_input(self):
        size = gr.sizeof_gr_complex if self.options.iq else gr.sizeof_float
        self.input = blocks.file_source(size, self.options.rawfile, False, 0, 0)

    def setup_rawint16_input(self):
        self.input_int16 = blocks.file_source(gr.sizeof_short, self.options.rawint16, False, 0, 0)
        self.setup_input_int16()

    def setup_audio_input(self):
        self.audio_source = audio.source(self.options.samp_rate, self.options.audio, True)
        if self.options.iq:
            self.float_to_complex = blocks.float_to_complex(1)
            self.connect((self.audio_source, 0), (self.float_to_complex, 0))
            self.connect((self.audio_source, 1), (self.float_to_complex, 1))
            self.input = self.float_to_complex
        else:
            self.input = self.audio_source
        
    def setup_udp_input(self):
        if self.options.udp_raw:
            size = gr.sizeof_gr_complex if self.options.iq else gr.sizeof_float
            self.input = blocks.udp_source(size, self.options.udp_ip, self.options.udp_port, 1472, False)
        else:
            self.input_int16 = blocks.udp_source(gr.sizeof_short, self.options.udp_ip, self.options.udp_port, 1472, False)
            self.setup_input_int16()

    def setup_input_int16(self):
        self.short_to_float = blocks.short_to_float(1, 32767)
        self.connect(self.input_int16, self.short_to_float)
        if self.options.iq:
            self.deinterleave = blocks.deinterleave(gr.sizeof_float, 1)
            self.float_to_complex = blocks.float_to_complex(1)
            self.connect(self.short_to_float, self.deinterleave)
            self.connect((self.deinterleave, 0), (self.float_to_complex, 0))
            self.connect((self.deinterleave, 1), (self.float_to_complex, 1))
            self.input = self.float_to_complex
        else:
            self.input = self.short_to_float

    def setup_kiss_input(self):
        self.input = datasources.kiss_file_source(self.options.kiss_in)

def main():
    parser = argument_parser()
    if len(sys.argv) >= 2 and sys.argv[1] in ['--version', '--list_satellites']:
        options = parser.parse_args()
        if options.list_satellites:
            list_satellites()
        sys.exit(0)
    
    if len(sys.argv) <= 1 or sys.argv[1][0] == '-':
        parser.print_usage(file = sys.stderr)
        sys.exit(1)
    
    tb = gr_satellites_top_block(parser)

    def sig_handler(sig=None, frame=None):
        tb.stop()
        tb.wait()
        sys.exit(0)

    signal.signal(signal.SIGINT, sig_handler)
    signal.signal(signal.SIGTERM, sig_handler)

    tb.start()
    tb.wait()
        
if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2018 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import numpy as np
from gnuradio import gr
import pmt

from .eseo_line_decoder import reflect_bytes

class eseo_packet_crop(gr.basic_block):
    """
    docstring for block eseo_packet_crop
    """
    def __init__(self, drop_rs):
        gr.basic_block.__init__(self,
            name="eseo_packet_handler",
            in_sig=[],
            out_sig=[])

        self.drop_rs = drop_rs
        
        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('out'))

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print("[ERROR] Received invalid message type. Expected u8vector")
            return
        packet = bytes(pmt.u8vector_elements(msg))

        # find packet end marker
        idx = packet.find(b'\x7e\x7e')
        if idx == -1:
            return
        crop = idx if not self.drop_rs else idx - 16
        if crop < 0:
            return

        # reverse byte ordering
        packet = np.frombuffer(packet[:crop], dtype = 'uint8')
        packet = np.packbits(reflect_bytes(np.unpackbits(packet)))
        packet = bytes(packet)

        self.message_port_pub(pmt.intern('out'),  pmt.cons(pmt.car(msg_pmt), pmt.init_u8vector(len(packet), packet)))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later

# This contains code by Peter Horvath <hp@hvt.bme.hu>
# (source: private communication)

from gnuradio import gr
import pmt

import json
import requests
from requests.auth import HTTPBasicAuth

class bme_submitter(gr.basic_block):
    """
    Submits telemetry to https://gnd.bme.hu:8080/
    """
    def __init__(self, user, password, satellite):
        gr.basic_block.__init__(self,
            name="bme_submitter",
            in_sig=[],
            out_sig=[])
        self.satellite = satellite

        self.authenticate(user, password)
        
        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)

    def authenticate(self, user, password):
        self.auth_token = None
        rauth = requests.post('https://gnd.bme.hu:8080/api/tokens', auth=HTTPBasicAuth(user, password), timeout=10)
        if rauth.status_code == 200:
            # we hit the jackpot, let's use the token obtained in the authentication header
            auth_resp = rauth.json()
            self.auth_token = auth_resp['token'] # the token is valid for 60 minutes, after 45 a new one can be requested
        elif rauth.status_code == 401:
            # unauthorized (wrong credentials)
            print('Wrong credentials, have you registered at https://gnd.bme.hu:8080/ ?')
        else:
            print(f'Authentication failed, error code = {rauth.status_code}')

    def putPacket(self, frame):
        if self.auth_token is None:
            print('Not uploading packet to BME, as we are not authenticated')
            return
        packets = [{'satellite': self.satellite, 'packet': frame.hex().upper()}]
        auth_header = {'Authorization': 'Bearer ' + self.auth_token}
        rpacket = requests.post('https://gnd.bme.hu:8080/api/packets/bulk', json={'packets':packets}, headers=auth_header, timeout=10)
        packet_resp = rpacket.json()
        if rpacket.status_code == 200:
            uploaded_packets = packet_resp["results"]
            for p in uploaded_packets:
                if 'error' in p:
                    print('Checksum error')
        else:
            print("Packet upload failed, token might have expired!")
            
    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print("[ERROR] Received invalid message type. Expected u8vector")
            return

        frame = bytes(pmt.u8vector_elements(msg))
        self.putPacket(frame)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import yaml
import pathlib
import string

class YAMLError(Exception):
    def __init__(self, message):
        self.message = message

_default_path = pathlib.Path(__file__).parent

class SatYAML:
    def __init__(self, path = _default_path):
        self._path = pathlib.Path(path)

    modulations = ['AFSK', 'FSK', 'BPSK', 'BPSK Manchester', 'DBPSK', 'DBPSK Manchester', 'FSK subaudio']
    framings = ['AX.25', 'AX.25 G3RUH', 'AX100 ASM+Golay', 'AX100 Reed Solomon',
                '3CAT-1', 'Astrocast FX.25 NRZ-I', 'Astrocast FX.25 NRZ',
                'AO-40 FEC', 'AO-40 FEC short', 'AO-40 uncoded', 'TT-64', 'ESEO', 'Lucky-7',
                'Reaktor Hello World', 'S-NET', 'Swiatowid', 'NuSat', 'K2SAT',
                'CCSDS Reed-Solomon', 'CCSDS Concatenated',
                'LilacSat-1', 'AAUSAT-4', 'NGHam', 'NGHam no Reed Solomon', 'SMOG-P RA',
                'SMOG-P Signalling', 'OPS-SAT', 'U482C', 'UA01', 'SALSAT',
                'Mobitex', 'Mobitex-NX', 'FOSSASAT', 'AISTECHSAT-2', 'AALTO-1',
                'Grizu-263A', 'IDEASSat', 'YUSAT', 'AX5043']
    transports = ['KISS', 'KISS no control byte', 'KISS KS-1Q']
    top_level_words = ['name', 'alternative_names', 'norad', 'telemetry_servers', 'data', 'transports', 'transmitters']
    
    def check_yaml(self, yml):
        d = self.get_yamldata(yml)
        for word in d:
            if word not in self.top_level_words:
                raise YAMLError(f'Unknown word {word} in {yml}')
        if 'name' not in d:
            raise YAMLError(f'Missing name field in {yml}')
        if 'norad' not in d:
            raise YAMLError(f'Missing norad field in {yml}')
        if type(d['norad']) is not int:
            raise YAMLError(f'NORAD field does not contain a number in {yml}')
        if 'telemetry_servers' in d:
            for server in d['telemetry_servers']:
                if server not in ['SatNOGS', 'FUNcube', 'PWSat', 'BME'] and\
                  not server.startswith('HIT ') and not server.startswith('SIDS '):
                    raise YAMLError(f'Unknown telemetry server {server}')
        if 'data' not in d:
            raise YAMLError(f'Missing data field in {yml}')
        if 'transports' in d:
            for key, transport in d['transports'].items():
                if 'protocol' not in transport:
                    raise YAMLError(f'Missing protocol field in transport {key}')
                if transport['protocol'] not in self.transports:
                    raise YAMLError(f'Unknown protocol field in transport {key}')
        if 'transmitters' not in d:
            raise YAMLError(f'Missing transmitters field in {yml}')
        for key, transmitter in d['transmitters'].items():
            if 'frequency' not in transmitter:
                raise YAMLError(f'Missing frequency field in {key} in {yml}')
            if type(transmitter['frequency']) not in [float, int]:
                raise YAMLError(f'Frequency field does not contain a float in {key} in {yml}')
            if 'modulation' not in transmitter:
                raise YAMLError(f'Missing modulation field in {key} in {yml}')
            if transmitter['modulation'] not in self.modulations:
                raise YAMLError(f'Unknown modulation in {key} in {yml}')
            if 'baudrate' not in transmitter:
                raise YAMLError(f'Missing baudrate field in {key} in {yml}')
            if type(transmitter['baudrate']) not in [float, int]:
                raise YAMLError(f'Baudrate field does not contain a float in {key} in {yml}')
            if transmitter['modulation'] == 'AFSK':
                if 'af_carrier' not in transmitter:
                    raise YAMLError(f'Missing af_carrier field for AFSK in {key} in {yml}')
                if 'deviation' not in transmitter:
                    raise YAMLError(f'Missing deviation field for AFSK in {key} in {yml}')
            if 'af_carrier' in transmitter and type(transmitter['af_carrier']) not in [float, int]:
                raise YAMLError(f'af_carrier field does not contain a float in {key} in {yml}')
            if 'deviation' in transmitter and type(transmitter['deviation']) not in [float, int]:
                raise YAMLError(f'Deviation field does not contain a float in {key} in {yml}')
            if 'framing' not in transmitter:
                raise YAMLError(f'Missing framing field in {key} in {yml}')
            if transmitter['framing'] not in self.framings:
                raise YAMLError(f'Unknown framing in {key} in {yml}')
            if transmitter['framing'] in ['CCSDS Reed-Solomon', 'CCSDS Concatenated']:
                if 'RS basis' not in transmitter:
                    raise YAMLError(f'No RS basis field in {key} in {yml}')
                if 'precoding' in transmitter and transmitter['precoding'] != 'differential':
                    raise YAMLError(f'Invalid precoding value {transmitter["precoding"]} for {key} in {yml}')
                if transmitter['RS basis'] not in ['conventional', 'dual']:
                    raise YAMLError(f'Invalid RS basis value {transmitter["RS basis"]} for {key} in {yml}')
                if 'RS interleaving' in transmitter and type(transmitter['RS interleaving']) is not int:
                    raise YAMLError(f'RS interleaving does not contain an int in {key} in {yml}')
                if 'scrambler' in transmitter and transmitter['scrambler'] not in ['CCSDS', 'none']:
                    raise YAMLError(f'Invalid scrambler value {transmitter["scrambler"]} for {key} in {yml}')
                if 'convolutional' in transmitter and transmitter['convolutional'] not in \
                  ['CCSDS', 'NASA-DSN', 'CCSDS uninverted', 'NASA-DSN uninverted']:
                    raise YAMLError(f'Invalid convolutional value {transmitter["convolutional"]} for {key} in {yml}')
            if 'data' not in transmitter and 'transports' not in transmitter:
                raise YAMLError(f'No data or transport field in {key} in {yml}')
            if 'data' in transmitter:
                for dd in transmitter['data']:
                    if dd not in d['data']:
                        raise YAMLError(f'Data entry {dd} used in {key} is not defined in data field in {yml}')
            if 'transport' in transmitter:
                for t in transmitter['transports']:
                    if t not in d['transports']:
                        raise YAMLError(f'Transport entry {t} used in {key} is not defined in transports field in {yml}')
            if 'additional_data' in transmitter:
                for k, dd in transmitter['additional_data'].items():
                    if dd not in d['data']:
                        raise YAMLError(f'Additional data entry {dd} used in {key} is not defined in data field in {yml}')
            
    def load_all_yaml(self):
        return [self.get_yamldata(f) for f in self.yaml_files()]
            
    def yaml_files(self):
        return self._path.glob('*.yml')

    def get_yamldata(self, yml):
        with open(yml, encoding = 'utf-8') as f:
            return yaml.safe_load(f)

    def _get_satnames(self, yml):
        d = self.get_yamldata(yml)
        alternative = d.get('alternative_names', [])
        if alternative is None:
            alternative = []
        return [d['name']] + alternative
        
    def _get_satnorad(self, yml):
        d = self.get_yamldata(yml)
        return d['norad']

    def _canonical_name(self, name):
        """Perform some substitutions in order to match names which are loosely equal"""
        return ''.join([a for a in name.casefold()
                        if a not in string.punctuation + string.whitespace])
    
    def search_name(self, name):
        name = self._canonical_name(name)
        for yml in self.yaml_files():
            satnames = [self._canonical_name(n) for n in self._get_satnames(yml)]
            if name in satnames:
                return self.get_yamldata(yml)
        raise ValueError('satellite not found')

    def search_norad(self, norad):
        for yml in self.yaml_files():
            if norad == self._get_satnorad(yml):
                return self.get_yamldata(yml)
        raise ValueError('satellite not found')

yamlfiles = SatYAML()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2018 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from construct import *
import datetime

class AffineAdapter(Adapter):
    def __init__(self, c, a, *args, **kwargs):
        self.c = c
        self.a = a
        return Adapter.__init__(self, *args, **kwargs)
    def _encode(self, obj, context, path = None):
        return int(round(obj * self.c + self.a))
    def _decode(self, obj, context, path = None):
        return (float(obj) - self.a)/ self.c

class LinearAdapter(AffineAdapter):
    def __init__(self, c, *args, **kwargs):
        return AffineAdapter.__init__(self, c, 0, *args, **kwargs)

class UNIXTimestampAdapter(Adapter):
    def _encode(self, obj, context, path = None):
        return round(obj.timestamp())
    def _decode(self, obj, context, path = None):
        return datetime.datetime.utcfromtimestamp(obj)

class TableAdapter(Adapter):
    def __init__(self, table, *args, **kwargs):
        self.table = table
        return Adapter.__init__(self, *args, **kwargs)
    def _encode(self, obj, context, path = None):
        try:
            return self.table.index(obj)
        except ValueError:
            return None
    def _decode(self, obj, context, path = None):
        try:
            return self.table[obj]
        except IndexError:
            return None




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2017,2020 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import numpy
from gnuradio import gr
import pmt
import datetime
import urllib.request, urllib.parse, urllib.error

def parse_time(time):
    try:
        return datetime.datetime.fromisofromat(time)
    except AttributeError:
        # workaround for Python version <3.7
        # which doesn't have fromisoformat()
        return datetime.datetime.strptime(time, '%Y-%m-%dT%H:%M:%S')

class submit(gr.basic_block):
    """
    docstring for block submit
    """
    def __init__(self, url, noradID, source, longitude, latitude, initialTimestamp):
        gr.basic_block.__init__(self,
            name="submit",
            in_sig=[],
            out_sig=[])

        self.url = url
        self.request = { 'noradID': noradID,\
                         'source': source,\
                         'locator': 'longLat',\
                         'longitude': str(abs(longitude)) + ('E' if longitude >= 0 else 'W'),\
                         'latitude': str(abs(latitude)) + ('N' if latitude >= 0 else 'S'),\
                         'version': '1.6.6' }
        self.initialTimestamp = parse_time(initialTimestamp) \
            if initialTimestamp != '' else None
        self.startTimestamp = datetime.datetime.utcnow()
        
        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        

    def handle_msg(self, msg_pmt):
        # check that callsign and QTH have been entered
        if self.request['source'] == '':
            return
        if self.request['longitude'] == 0.0 and self.request['latitude'] == 0.0:
            return

        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print("[ERROR] Received invalid message type. Expected u8vector")
            return

        self.request['frame'] = bytes(pmt.u8vector_elements(msg)).hex().upper()
        
        now = datetime.datetime.utcnow()
        timestamp = now - self.startTimestamp + self.initialTimestamp \
          if self.initialTimestamp else now
        self.request['timestamp'] = timestamp.isoformat()[:-3] + 'Z'

        params = urllib.parse.urlencode(self.request)
        try:
            f = urllib.request.urlopen('{}?{}'.format(self.url, params),
                                           data = bytes(params, encoding = 'ascii'))
        except Exception as e:
            print('Error while submitting telemetry:', e)
            return
        reply = f.read()
        code = f.getcode()
        if code < 200 or code >= 300:
            print("Server error while submitting telemetry")
            print("Reply:")
            print(reply)
            print("URL:", f.geturl())
            print("HTTP code:", f.getcode())
            print("Info:")
            print(f.info())
        f.close()
        



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from pathlib import Path
import configparser

def open_config():
    """
    Opens and returns the gr-satellites config .ini file

    If the ini file does not exist, it creates a default one
    """
    config_dir = Path.home() / '.gr_satellites'
    config_filename = config_dir / 'config.ini'

    if not config_dir.exists():
        Path.mkdir(config_dir)

    if not config_filename.exists():
        return write_default_config(config_filename)

    config = configparser.ConfigParser()
    config.read(config_filename)
    return config

def write_default_config(file):
    """
    Writes a default configuration and returns the
    default configuration values.

    Args:
        file: the filename to write to
    """
    config = configparser.ConfigParser()
    config['Groundstation'] = {
        'callsign' : '',
        'latitude' : 0,
        'longitude' : 0,
        'submit_tlm' : 'yes',
    }

    config['FUNcube'] = {
        'site_id' : '',
        'auth_code' : '',
    }
    
    config['PW-Sat2'] = {
        'credentials_file' : '',
    }

    config['BME'] = {
        'user' : '',
        'password' : '',
    }

    with open(file, 'w') as f:
        config.write(f)

    return config



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, digital, blocks, fec
from ... import descrambler308
from ... import k2sat_deframer as deframer
from ...hier.sync_to_pdu_packed import sync_to_pdu_packed
from ...utils.options_block import options_block

_syncword = '0101010101111110'

class k2sat_deframer(gr.hier_block2, options_block):
    """
    Hierarchical block to deframe the K2SAT custom image protocol

    The input is a float stream of soft symbols. The output are PDUs
    with frames.

    Args:
        syncword_threshold: number of bit errors allowed in syncword (int)
        options: Options from argparse
    """
    def __init__(self, syncword_threshold = None, options = None):
        gr.hier_block2.__init__(self, "k2sat_deframer",
            gr.io_signature(1, 1, gr.sizeof_float),
            gr.io_signature(0, 0, 0))
        options_block.__init__(self, options)
        
        self.message_port_register_hier_out('out')

        if syncword_threshold is None:
            syncword_threshold = self.options.syncword_threshold

        # For QPKS 90º abiguity
        self.deinterleave = blocks.deinterleave(gr.sizeof_float, 1)
        self.invert = blocks.multiply_const_ff(-1, 1)
        self.interleave = blocks.interleave(gr.sizeof_float, 1)
        
        self.cc0 = fec.cc_decoder.make(80, 7, 2, [79, 109], 0, -1, fec.CC_STREAMING, False)
        self.cc1 = fec.cc_decoder.make(80, 7, 2, [79, 109], 0, -1, fec.CC_STREAMING, False)
        self.viterbi0 = fec.extended_decoder(decoder_obj_list = self.cc0,\
                                             threading = None,\
                                             ann = None,
                                             puncpat = '11')
        self.viterbi1 = fec.extended_decoder(decoder_obj_list = self.cc1,\
                                             threading = None,\
                                             ann = None,
                                             puncpat = '11')
        self.diff0 = digital.diff_decoder_bb(2)
        self.diff1 = digital.diff_decoder_bb(2)
        self.scrambler0 = descrambler308()
        self.scrambler1 = descrambler308()
        self.deframer0 = sync_to_pdu_packed(packlen = 2200,\
                                        sync = _syncword,\
                                        threshold = syncword_threshold)
        self.deframer1 = sync_to_pdu_packed(packlen = 2200,\
                                        sync = _syncword,\
                                        threshold = syncword_threshold)
        self.deframer = deframer()
        
        self.connect(self, self.viterbi0, self.diff0, self.scrambler0, self.deframer0)
        self.connect(self, self.deinterleave)
        self.connect((self.deinterleave, 0), (self.interleave, 1))
        self.connect((self.deinterleave, 1), self.invert, (self.interleave, 0))
        self.connect(self.interleave, self.viterbi1, self.diff1, self.scrambler1, self.deframer1)
        
        self.msg_connect((self.deframer0, 'out'), (self.deframer, 'in'))
        self.msg_connect((self.deframer1, 'out'), (self.deframer, 'in'))
        self.msg_connect((self.deframer, 'out'), (self, 'out'))

    _default_sync_threshold = 0
        
    @classmethod
    def add_options(cls, parser):
        """
        Adds K2SAT deframer specific options to the argparse parser
        """
        parser.add_argument('--syncword_threshold', type = int, default = cls._default_sync_threshold, help = 'Syncword bit errors [default=%(default)r]')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import pathlib

class File:
    """
    Class to hold housekeeping data about a file being received
    """
    def __init__(self, path):
        """
        Builds a new file, initializing housekeeping data

        Args:
            path: path of the file (pathlib.Path)
        """
        self.path = path
        mode = 'r+b' if path.exists() else 'wb'
        self.f = open(path, mode)
        self.broken = False
        self.expected_seq = 0
        self.write_pointer = 0
        self.size = None
        self.chunks = None

class FileReceiver:
    """
    Class to reassemble files transmitted in chunks

    This implements the generic framework. Specific protocols should
    inherit from this class and implement some functions
    """
    def __init__(self, path, verbose = False):
        """
        Builds a new FileReceiver

        Args:
            path: directory where files should be stored (str or pathlib.Path)
            verbose: verbose reporting of events (bool)
        """
        self._files = dict()
        self._current_file = None
        self._next_fid = 0
        self._path = pathlib.Path(path)
        self._verbose = verbose
        self._name = 'FileReceiver'

    def file_id(self, chunk):
        """
        Returns the file ID of a given chunk

        If the file cannot be identified from a chunk, it should
        return None.

        The default implementation returns chunk.file_id if available
        else None
        
        Args:
            chunk: a file chunk (parsed by parse_chunk())
        """
        return getattr(chunk, 'file_id', None)

    def chunk_sequence(self, chunk):
        """
        Returns the sequence number of a given chunk
        
        The sequence number should increase from 0. If the chunk number
        cannot be identified from a chunk, or if chunk_offset is used
        instead, it should return None.

        The default implementation returns chunk.sequence if available
        else None

        Args:
            chunk: a file chunk (parsed by parse_chunk())
        """
        return getattr(chunk, 'sequence', None)

    def chunk_size(self):
        """
        Returns the size of a chunk if all chunks (except possibly
        the last one) have the same fixed size, None if not
        """
        return None

    def chunk_offset(self, chunk):
        """
        Returns the offset in bytes of a given chunk within the file

        This should return None if it is not possible to determine
        the chunk offset from the chunk.

        The default implementation first returns chunk.offset if available.
        If not, it tries to use chunk_sequence() and chunk_size() to
        determine the offset.

        Args:
            chunk: a file chunk (parsed by parse_chunk())
        """
        off = getattr(chunk, 'offset', None)
        if off is not None:
            return off
        s = self.chunk_size()
        i = self.chunk_sequence(chunk)
        if s is not None and i is not None:
            return s * i
        else:
            return None

    def chunk_data(self, chunk):
        """
        Returns the file data in a given chunk

        The default implementation returns chunk.data.

        Args:
            chunk: a file chunk (parsed by parse_chunk())
        Returns:
            bytes
        """
        return chunk.data

    def file_chunks(self, chunk):
        """
        Returns the total number of chunks in the file

        The default implementation returns chunk.chunks if available
        else None

        Args:
            chunk: a file chunk (parsed by parse_chunk())
        """
        return getattr(chunk, 'chunks', None)

    def file_size(self, chunk):
        """
        Returns the total number of bytes in the file

        The default implementation returns chunk.filesize if available
        else None

        Args:
            chunk: a file chunk (parsed by parse_chunk())
        """
        return getattr(chunk, 'filesize', None)

    def is_last_chunk(self, chunk):
        """
        Returns whether this is the last chunk in the file

        The default implementation returns None

        Args:
            chunk: a file chunk (parsed by parse_chunk())
        Returns:
            True if this is is the last chunk, False if this is not
            the last chunk, None if it is unknown whether this is the
            last chunk.
        """
        return None
    
    def parse_chunk(self, chunk):
        """
        Parses the chunk into an object which is easier to handle

        The default implementation does nothing.
        A return of None means the chunk is invalid and should not
        be processed.

        Args:
            chunk: a file chunk (bytes)
        """
        return chunk

    def filename(self, fid):
        """
        Generates a filename based on the file id

        The default implementation uses the file id as filename
        
        Args:
           fid: file id (usually int)
        """
        return str(fid)

    def on_completion(self, f):
        """
        Function to be called on file completion

        The user should overload this function if some action
        needs to be done after a file is complete

        Args:
            f: the file just completed (File)
        """
    
    def _new_file(self, fid):
        """
        Creates a new file

        Args:
            fid: file id of the new file (usually int)
        Returns:
            the new file
        """
        f = File(self._path / self.filename(fid))
        self._files[fid] = f
        return f

    def _fill_file_data(self, f, chunk):
        """
        Tries to fill some file data from the chunk

        Args:
            f: a File
            chunk: a file chunk (parsed by parse_chunk())
        """
        if f.size is None:
            f.size = self.file_size(chunk)
        if f.chunks is None:
            f.chunks = self.file_chunks(chunk)

    def log(self, message):
        if self._verbose:
            print(f'{self._name}: {message}')
            
    def push_chunk(self, chunk):
        """
        Processes a new chunk

        The user should call this function whenever a new chunk
        is received.

        Args:
            chunk: the file chunk (bytes)
        """
        chunk = self.parse_chunk(chunk)
        if chunk is None:
            return
        
        # find file
        # TODO: logic about new file when file_id == None
        fid = self.file_id(chunk)
        if fid is None:
            fid = self._current_file
        if fid is None:
            fid = self._next_fid
            self._next_fid += 1
        try:
            f = self._files[fid]
        except KeyError:
            self.log(f'new file {fid}')
            f = self._new_file(fid)

        seq = self.chunk_sequence(chunk)

        # detect a new file if sequence has decreased
        if self.file_id(chunk) is None and seq is not None and seq < f.expected_seq:
            # f is finished
            self.log(f'file {fid} is finished')
            fid = self._next_fid
            self._next_fid += 1
            f = self._new_file(fid)
            self.log(f'receiving new file {fid}')
        self._current_file = fid

        self._fill_file_data(f, chunk)

        if seq is not None:
            self.log(f'received sequence {seq} for file {fid}')

        if f.broken:
            # file is broken, nothing can be done with the chunk
            return

        # try to determine offset
        offset = self.chunk_offset(chunk)
        if seq is None and offset is not None:
            self.log(f'received offset {offset} for file {fid}')
        if offset is None:
            # check sequence continuity
            if seq is None:
                raise Exception('FileReceiver unable to compute chunk_offset() nor chunk_sequence()')
            if seq != f.expected_seq:
                self.log(f'received sequence {seq} != expected sequence {f.expected_seq}. File reception is broken.')
                f.broken = True
                return
            offset = f.write_pointer
            
        data = self.chunk_data(chunk)
        new_write_pointer = offset + len(data)

        # check offset within size
        if f.size is not None and new_write_pointer > f.size:
            self.log(f'invalid offset {offset} (chunk size is {len(data)}, file size is {f.size})')
            return

        # write chunk at offset
        f.f.seek(offset)
        f.f.write(data)
        f.f.flush()

        # update pointers
        f.write_pointer = new_write_pointer
        f.expected_seq = seq + 1 if seq is not None else None

        # check if file is complete
        if (f.size is not None and f.write_pointer >= f.size) or \
          (f.chunks is not None and f.expected_seq >= f.chunks) or \
          self.is_last_chunk(chunk) is True:
          self.log(f'file {fid} complete')
          self.on_completion(f)
          self._current_file = None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2017 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import numpy
from gnuradio import gr
import pmt
import struct

from . import crc32c

class append_crc32c(gr.basic_block):
    """
    docstring for block append_crc32c
    """
    def __init__(self, include_header):
        gr.basic_block.__init__(self,
            name="append_crc32c",
            in_sig=[],
            out_sig=[])

        self.include_header = include_header
        
        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('out'))

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print("[ERROR] Received invalid message type. Expected u8vector")
            return
        packet = bytes(pmt.u8vector_elements(msg))
        crc = crc32c.crc(packet if self.include_header else packet[4:])
        packet += struct.pack(">I", crc)
        self.message_port_pub(pmt.intern('out'),
                              pmt.cons(pmt.PMT_NIL, pmt.init_u8vector(len(packet), packet)))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, blocks
from ... import pdu_to_kiss
from ...utils.options_block import options_block

class kiss_file_sink(gr.hier_block2, options_block):
    """
    Hierarchical block for KISS file output

    The input are PDUs with frames.

    These are saved to a KISS file.

    Args:
        file: output filename (string)
        append: append to file (bool)
        options: options from argparse
    """
    def __init__(self, file, append = False, options = None):
        gr.hier_block2.__init__(self, "kiss_file_sink",
            gr.io_signature(0, 0, 0),
            gr.io_signature(0, 0, 0))
        options_block.__init__(self, options)
        
        self.message_port_register_hier_in('in')

        initial_timestamp = getattr(self.options, 'start_time', '')
        
        self.kiss = pdu_to_kiss(include_timestamp = True,
                                initial_timestamp = initial_timestamp)
        self.pdu2tag = blocks.pdu_to_tagged_stream(blocks.byte_t, 'packet_len')
        self.filesink = blocks.file_sink(gr.sizeof_char, file, append)

        self.connect(self.pdu2tag, self.filesink)
        self.msg_connect((self, 'in'), (self.kiss, 'in'))
        self.msg_connect((self.kiss, 'out'), (self.pdu2tag, 'pdus'))

    @classmethod
    def add_options(cls, parser):
        """
        Adds KISS file sink specific options to the argparse parser
        """
        parser.add_argument('--start_time', type = str, default = '', help = 'Recording start timestamp')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import struct

from .imagereceiver import ImageReceiver

class ImageReceiverK2SAT(ImageReceiver):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._last_frame_count = None
        self._unwrap = 0
        self._first_header_frame_count = None

    def _first_header_pointer(self, chunk):
        return chunk[16 + 3]

    def _virtual_channel_frame_count(self, chunk):
        return chunk[16 + 2]

    def _unwrapped_frame_count(self, chunk):
        fc = self._virtual_channel_frame_count(chunk) + self._unwrap
        if self._last_frame_count is not None and fc < self._last_frame_count:
            self._unwrap += 256
            fc += 256
        self._last_frame_count = fc
        return fc
        
    def chunk_sequence(self, chunk):
        fh = self._first_header_pointer(chunk)
        if fh == 0x00:
            # first packet in image
            self._first_header_frame_count = self._unwrapped_frame_count(chunk)
            return 0
        return self._unwrapped_frame_count(chunk) - self._first_header_frame_count if \
          self._first_header_frame_count is not None else None

    def is_last_chunk(self, chunk):
        # 0xff indicates last packet in image
        return self._first_header_pointer(chunk) == 0xff

    def chunk_data(self, chunk):
        return chunk[20:-1]

    def filename(self, fid):
        return f'{fid}.jpg'

    def parse_chunk(self, chunk):
        return chunk if len(chunk) > 16 + 4 + 1 else None

k2sat = ImageReceiverK2SAT



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import numpy
from gnuradio import gr
import pmt

class swiatowid_packet_split(gr.basic_block):
    """
    docstring for block swiatowid_packet_split
    """
    def __init__(self):
        gr.basic_block.__init__(self,
            name="swiatowid_packet_split",
            in_sig=[],
            out_sig=[])

        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('out'))

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print("[ERROR] Received invalid message type. Expected u8vector")
            return
        packet = pmt.u8vector_elements(msg)

        for j in range(0, len(packet) - 2, 58): # -2 is included for the case when the packet carriers a CRC-16
            piece = packet[j:j+58]
            self.message_port_pub(pmt.intern('out'),
                                  pmt.cons(pmt.car(msg_pmt), pmt.init_u8vector(len(piece), piece)))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import struct

from .imagereceiver import ImageReceiver

class ImageReceiver1KUNSPF(ImageReceiver):
    def chunk_sequence(self, chunk):
        return struct.unpack('>H', chunk[4:6])[0]

    def chunk_data(self, chunk):
        return chunk[6:-4]

    def filename(self, fid):
        return f'{fid}.jpg'

    def parse_chunk(self, chunk):
        if len(chunk) != 138:
            return None
        else:
            return chunk

sat_1kuns_pf = ImageReceiver1KUNSPF



# -*- coding: utf-8 -*-

#
# SPDX-License-Identifier: GPL-3.0
#
# GNU Radio Python Flow Graph
# Title: RMS AGC
# Author: Daniel Estevez
# Description: AGC using RMS
# GNU Radio version: 3.8.0.0

from gnuradio import blocks
from gnuradio import gr
from gnuradio.filter import firdes
import sys
import signal




class rms_agc_f(gr.hier_block2):
    def __init__(self, alpha=1e-2, reference=0.5):
        gr.hier_block2.__init__(
            self, "RMS AGC",
                gr.io_signature(1, 1, gr.sizeof_float*1),
                gr.io_signature(1, 1, gr.sizeof_float*1),
        )

        ##################################################
        # Parameters
        ##################################################
        self.alpha = alpha
        self.reference = reference

        ##################################################
        # Blocks
        ##################################################
        self.blocks_rms_xx_0 = blocks.rms_ff(alpha)
        self.blocks_multiply_const_vxx_0 = blocks.multiply_const_ff(1.0/reference)
        self.blocks_divide_xx_0 = blocks.divide_ff(1)
        self.blocks_add_const_vxx_0 = blocks.add_const_ff(1e-20)



        ##################################################
        # Connections
        ##################################################
        self.connect((self.blocks_divide_xx_0, 0), (self, 0))
        self.connect((self.blocks_add_const_vxx_0, 0), (self.blocks_divide_xx_0, 1))
        self.connect((self.blocks_multiply_const_vxx_0, 0), (self.blocks_add_const_vxx_0, 0))
        self.connect((self.blocks_rms_xx_0, 0), (self.blocks_multiply_const_vxx_0, 0))
        self.connect((self, 0), (self.blocks_divide_xx_0, 0))
        self.connect((self, 0), (self.blocks_rms_xx_0, 0))

    def get_alpha(self):
        return self.alpha

    def set_alpha(self, alpha):
        self.alpha = alpha
        self.blocks_rms_xx_0.set_alpha(self.alpha)

    def get_reference(self):
        return self.reference

    def set_reference(self, reference):
        self.reference = reference
        self.blocks_multiply_const_vxx_0.set_k(1.0/self.reference)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020 jgromes <gromes.jan@gmail.com>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from construct import *
from ..adapters import LinearAdapter
from .fossasat import *

fossasat_2 = Struct(
    'callsign' / Const(b"FOSSASAT-2"),
    'func_id' / FuncId,
    'opt_data_len' / Optional(Int8ul),
    'payload' / Switch(this.func_id, {
        'RESP_PONG' : Pong,
        'RESP_REPEATED_MESSAGE' : RepeatedMessage,
        'RESP_REPEATED_MESSAGE_CUSTOM' : RepeatedMessage,
        'RESP_SYSTEM_INFO' : SystemInfo,
        'RESP_PACKET_INFO' : PacketInfo,
        'RESP_STATISTICS' : Statistics,
        'RESP_FULL_SYSTEM_INFO' : FullSystemInfo,
        'RESP_STORE_AND_FORWARD_ASSIGNED_SLOT' : StoreAndForwardAssigned,
        'RESP_FORWARDED_MESSAGE' : ForwardedMessage,
        'RESP_PUBLIC_PICTURE' : CameraPicture,
        'RESP_DEPLOYMENT_STATE' : DeploymentState,
        'RESP_CAMERA_STATE' : CameraState,
        'RESP_RECORDED_IMU' : RecordedIMU,
        'RESP_MANUAL_ACS_RESULT' : ManualACSResult,
        'RESP_GPS_LOG' : GPSLog,
        'RESP_GPS_LOG_STATE' : GPSLogState,
        'RESP_FLASH_CONTENTS' : FlashContents,
        'RESP_CAMERA_PICTURE' : CameraPicture,
        'RESP_CAMERA_PICTURE_LENGTH' : CameraPictureLength,
        'RESP_GPS_COMMAND_RESPONSE' : GPSCommandResponse,
        'RESP_ACKNOWLEDGE' : Acknowledge,
    }, default = HexDump(GreedyBytes)),
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2018-2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from construct import *
from .ax25 import Header
import datetime

# An adapted version of Space Packet primary header
# where the APID is broken down as used by the PicSat mission
PrimaryHeader = BitStruct('ccsds_version' / BitsInteger(3),
                          'packet_type' / Flag,
                          'secondary_header_flag' / Flag,
                          'process_id' / BitsInteger(4),
                          'level_flag' / Flag,
                          'payload_flag' / Flag,
                          'packet_category' / BitsInteger(5),
                          'sequence_flag' / BitsInteger(2),
                          'packet_id' / BitsInteger(14),
                          'data_length' / BitsInteger(16))

class TimeAdapter(Adapter):
    def _encode(self, obj, context, path = None):
        d = obj - datetime.datetime(1970, 1, 1)
        return Container(days = d.days, milliseconds = d.seconds * 1000 + d.microseconds / 1000)
    def _decode(self, obj, context, path = None):
        return datetime.datetime(1970, 1, 1) + datetime.timedelta(days=obj.days, seconds=obj.milliseconds/1000, microseconds=(obj.milliseconds%1000)*1000)

SecondaryHeaderTM = TimeAdapter(Struct('days' / Int16ub, 'milliseconds' / Int32ub))

SecondaryHeaderTC = BitStruct('req_ack_reception' / Flag,
                              'req_fmt_reception' / Flag,
                              'req_exe_reception' / Flag,
                              'telecommand_id' / BitsInteger(10),
                              'emitter_id' / BitsInteger(3),
                              'signature' / Bytes(16))

AntStatus = Struct(
    'undeployed' / Flag,
    'timeout' / Flag,
    'deploying' / Flag)

BeaconA = BitStruct(
    Padding(1),
    'solar_panel_error_flags' / Flag[5],
    'i_adcs_get_attitude_error' / Flag,
    'i_adcs_get_status_register_error' / Flag,
    Padding(1),
    'fram_enable_error_flag' / Flag,
    'ants_error_flag' / Flag[2],
    'trxvu_tx_error_flag' / Flag,
    'trxvu_rx_error_flag' / Flag,
    'obc_supervisor_error_flag' / Flag,
    'gom_eps_error_flag' / Flag,
    'ant1_status_b' / AntStatus,
    Padding(1),
    'ant2_status_b' / AntStatus,
    'ignore_flag_ants_b_status' / Flag,
    'ant3_status_b' / AntStatus,
    Padding(1),
    'ant4_status_b' / AntStatus,
    'armed_ants_b_status' / Flag,
    'ant1_status_a' / AntStatus,
    Padding(1),
    'ant2_status_a' / AntStatus,
    'ignore_flag_ants_a_status' / Flag,
    'ant3_status_a' / AntStatus,
    Padding(1),
    'ant4_status_a' / AntStatus,
    'armed_ants_a_status' / Flag)
    
BeaconB = Struct(
    'solar_panel_temps' / Int16ub[5],
    'ants_temperature' / Int16ub[2],
    'tx_trxvu_hk_current' / Int16ub,
    'tx_trxvu_hk_forwardpower' / Int16ub,
    'tx_trxvu_tx_reflectedpower' / Int16ub,
    'tx_trxvu_hk_pa_temp' / Int16ub,
    'rx_trxvu_hk_pa_temp' / Int16ub,
    'rx_trxvu_hk_board_temp' / Int16ub,
    'eps_hk_temp_batts' / Int16sb,
    'eps_hk_batt_mode' / Int8ub,
    'eps_h_kv_batt' / Int8ub,
    'eps_hk_boot_cause' / Int32ub,
    'n_reboots_eps' / Int32ub,
    'n_reboots_obc' / Int32ub,
    'quaternions' / Float32b[4],
    'angular_rates' / Float32b[3])

BeaconC = BitStruct(
    Padding(12),
    'adcs_stat_flag_hl_op_tgt_cap' / Flag,
    'adcs_stat_flag_hl_op_tgt_track_fix_wgs84' / Flag,
    'adcs_stat_flag_hl_op_tgt_track_nadir' / Flag,
    'adcs_stat_flag_hl_op_tgt_track' / Flag,
    'adcs_stat_flag_hl_op_tgt_track_const_v' / Flag,
    'adcs_stat_flag_hl_op_spin' / Flag,
    Padding(1),
    'adcs_stat_flag_hl_op_sunp' / Flag,
    'adcs_stat_flag_hl_op_detumbling' / Flag,
    'adcs_stat_flag_hl_op_measure' / Flag,
    Padding(5),
    'adcs_stat_flag_datetime_valid' / Flag,
    Padding(1),
    'adcs_stat_flag_hl_op_safe' / Flag,
    'adcs_stat_flag_hl_op_idle' / Flag,
    Padding(1))

BeaconD = Struct(
    'up_time' / Int32ub,
    'last_fram_log_fun_err_code' / Int16sb,
    'last_fram_log_line_code' / Int16ub,
    'last_fram_log_file_crc_code' / Int32ub,
    'last_fram_log_counter' / Int16ub,
    'average_photon_count' / Int16ub,
    'sat_mode' / Int8ub,
    'tc_sequence_count' / Int16ub)

Beacon = Struct(Padding(3), BeaconA, BeaconB, BeaconC, BeaconD)

PayloadBeaconFlags = BitStruct(
    'hk_flag' / Flag,
    'cheatmode_flag' / Flag,
    'tec_flag' / Flag,
    'sensors_flag' / Flag,
    'hv_flag' / Flag,
    'dac_flag' / Flag,
    'interrupt_flag' / Flag,
    'diode_flag' / Flag)

PayloadBeacon = Struct(
    'message' / Bytes(29),
    'phot' / Int16ub,
    'mode' / Int8ub,
    'acqmode' / Int8ub,
    PayloadBeaconFlags,
    'proc_freq' / Int8ub,
    'volt5' / Int16ub,
    'amp5' / Int16ub,
    'amp3' / Int16ub,
    'volthv' / Int16ub,
    'amphv' / Int16ub,
    'temps' / Int16sb[4],
    'vitec' / Int16ub,
    'temp0' / Int16ub,
    'errortherm' / Int16ub,
    'vref' / Int16ub,
    'pitch' / Int16sb,
    'roll' / Int16sb,
    'yaw' / Int16sb)

picsat = Struct(
    'ax25_header' / Header,
    'primary_header' / PrimaryHeader,
    'secondary_header' / IfThenElse(lambda c: c.primary_header.packet_type, SecondaryHeaderTC, SecondaryHeaderTM),
    'packet' / Switch(lambda c: (c.primary_header.payload_flag, c.primary_header.packet_category), {
        (False,1) : Beacon,
        (True,7) : PayloadBeacon,
    }, default = Pass))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2021 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, digital
import pmt
import numpy as np

import struct

from ... import nrzi_decode
from ...hier.sync_to_pdu import sync_to_pdu
from ...utils.options_block import options_block
from ...check_crc16_ccitt_false import crc16_ccitt_false

# the 40 bit syncword is formed by 4130f000,
# which is the end of the AX.25 address of the first subpacket
# (including the subpacket counter),
# encoded as UART with 1 stop bit
_syncword = '0010000011000110000101111000010000000001'

class uart_decode(gr.basic_block):
    """
    Helper block to remove UART-like encoding

    The input is unpacked bits having 10 bits per byte consisting of a
    start bit (which should be 0 but we don't look at), an 8-bit byte
    in MSB-first order, and a stop bit (which should be 1 but we don't look at).
    The output consists of the 8-bit bytes as packed bytes.
    """
    def __init__(self):
        gr.basic_block.__init__(self,
            name="uart_decode",
            in_sig=[],
            out_sig=[])
        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('out'))

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print("[ERROR] Received invalid message type. Expected u8vector")
            return
        packet = np.array(pmt.u8vector_elements(msg), dtype = 'uint8')
        if packet.size % 10 != 0:
            print("[ERROR] Packet size is not a multiple of 10 bits")
            return
        packet = np.packbits(packet.reshape((-1,10))[:,1:-1])
        self.message_port_pub(pmt.intern('out'),
                              pmt.cons(pmt.PMT_NIL, pmt.init_u8vector(len(packet), packet)))

class extract_payload(gr.basic_block):
    """
    Helper block to throw away subpacket headers and extract CRC-protected payload

    This also checks the CRC
    """
    def __init__(self, verbose = False):
        gr.basic_block.__init__(self,
            name="extract_payload",
            in_sig=[],
            out_sig=[])
        self.verbose = verbose
        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('out'))

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print("[ERROR] Received invalid message type. Expected u8vector")
            return
        packet = np.array(pmt.u8vector_elements(msg), dtype = 'uint8')
        # put initial padding to account for the AX.25 header we've lost
        initial_padding = np.zeros(17, dtype = 'uint8')
        packet = np.concatenate((initial_padding, packet))
        if packet.size != 40 * 9:
            print("[ERROR] Invalid packet size")
            return
        packet = packet.reshape((9, 40))
        # save AX.25 header. we take it from the 2nd message
        header = packet[1,1:16]
        # drop AX.25 headers and final 0x7e
        packet = packet[:,17:-1].ravel()
        # drop final padding
        packet = packet[:-11]
        # check CRC
        crc = crc16_ccitt_false(packet[4:-2]) # do not include first 4 bytes
        if crc != struct.unpack('<H', packet[-2:])[0]:
            if self.verbose:
                print('CRC failed')
                return
        elif self.verbose:
            print('CRC OK')
        # drop CRC
        packet = packet[:-2]
        # put AX.25 header back
        packet = np.concatenate((header, packet))
        self.message_port_pub(pmt.intern('out'),
                              pmt.cons(pmt.PMT_NIL, pmt.init_u8vector(len(packet), packet)))

class ideassat_deframer(gr.hier_block2, options_block):
    """
    Hierarchical block to deframe IDEASSat ad-hoc UART-like protocol

    The input is a float stream of soft symbols. The output are PDUs
    with IDEASSat frames.

    Args:
        options: Options from argparse
    """
    def __init__(self, options = None):
        gr.hier_block2.__init__(self, "ideassat_deframer",
            gr.io_signature(1, 1, gr.sizeof_float),
            gr.io_signature(0, 0, 0))
        options_block.__init__(self, options)
        
        self.message_port_register_hier_out('out')

        self.slicer = digital.binary_slicer_fb()
        self.nrzi = nrzi_decode()
        # a length of 40 bytes will give at the end the two 0x7e HDLC flags
        # we do not allow syncword errors because this protocol is very brittle
        self.deframer = sync_to_pdu(packlen = 10 * (9*40 - 17),
                                    sync = _syncword,\
                                    threshold = 0)
        self.uart = uart_decode()
        self.payload = extract_payload(self.options.verbose_crc)

        self.connect(self, self.slicer, self.nrzi, self.deframer)
        self.msg_connect((self.deframer, 'out'), (self.uart, 'in'))
        self.msg_connect((self.uart, 'out'), (self.payload, 'in'))
        self.msg_connect((self.payload, 'out'), (self, 'out'))

    @classmethod
    def add_options(cls, parser):
        """
        Adds IDEASSat deframer specific options to the argparse parser
        """
        parser.add_argument('--verbose_crc', action = 'store_true', help = 'Verbose CRC')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2017 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import numpy
from gnuradio import gr
import pmt
import struct

class check_address(gr.basic_block):
    """
    docstring for block check_address
    """
    def __init__(self, address, direction):
        gr.basic_block.__init__(self,
            name="check_address",
            in_sig=[],
            out_sig=[])

        a = address.split('-')
        self.callsign = a[0]
        self.ssid = int(a[1]) if len(a) > 1 else None
        self.direction = direction
        
        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('ok'))
        self.message_port_register_out(pmt.intern('fail'))

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print("[ERROR] Received invalid message type. Expected u8vector")
            return
        packet = bytes(pmt.u8vector_elements(msg))

        # check packet length
        # an AX.25 header with 2 addresses, control and PID is 16 bytes
        if len(packet) < 16:
            self.message_port_pub(pmt.intern('fail'), msg_pmt)
            return

        if self.direction == 'to':
            address = packet[:7]
        else:
            address = packet[7:14]

        callsign = bytes([c >> 1 for c in address[:6]]).decode('ascii').rstrip(' ')
        ssid = (address[6] >> 1) & 0x0f

        if callsign != self.callsign or (self.ssid != None and ssid != self.ssid):
            # incorrect address
            self.message_port_pub(pmt.intern('fail'), msg_pmt)
        else:
            # correct address
            self.message_port_pub(pmt.intern('ok'), msg_pmt)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2017 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import numpy
from gnuradio import gr
import pmt

import datetime

class print_timestamp(gr.basic_block):
    """
    docstring for block print_timestamp
    """
    def __init__(self, tstamp_format='', count_packets=False):
        gr.basic_block.__init__(self,
            name="swap_crc",
            in_sig=[],
            out_sig=[])

        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('out'))
        self.count_packets = count_packets
        self.packet_counter = 0

        self.tstamp_format = tstamp_format

    def handle_msg(self, msg_pmt):
        if self.tstamp_format:
            timestamp = datetime.datetime.utcnow()
            print((timestamp.strftime(self.tstamp_format)))
        if self.count_packets:
            print('Packet number', self.packet_counter)
            self.packet_counter += 1
        
        self.message_port_pub(pmt.intern('out'), msg_pmt)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

'''
gr-satellites datasink components

The datasinks consume frames or packets in order to do something
useful with them (printing telemetry, reconstructing images,
storing to file, etc.)

The input to these blocks are PDUs with the frames.
'''

from .codec2_udp_sink import codec2_udp_sink
from .file_receiver import file_receiver
from .hexdump_sink import hexdump_sink
from .kiss_file_sink import kiss_file_sink
from .kiss_server_sink import kiss_server_sink
from .telemetry_submit import telemetry_submit
from .telemetry_parser import telemetry_parser



# -*- coding: utf-8 -*-

#
# SPDX-License-Identifier: GPL-3.0
#
# GNU Radio Python Flow Graph
# Title: Sync and create packed PDU
# Author: Daniel Estevez
# Description: Finds syncword and creates a PDU of fixed size and packed bytes
# GNU Radio version: 3.8.0.0

from gnuradio import blocks
from gnuradio import digital
from gnuradio import gr
from gnuradio.filter import firdes
import sys
import signal
import satellites
import numpy




class sync_to_pdu_packed(gr.hier_block2):
    def __init__(self, packlen=0, sync="00011010110011111111110000011101", threshold=4):
        gr.hier_block2.__init__(
            self, "Sync and create packed PDU",
                gr.io_signature(1, 1, gr.sizeof_char*1),
                gr.io_signature(0, 0, 0),
        )
        self.message_port_register_hier_out("out")

        ##################################################
        # Parameters
        ##################################################
        self.packlen = packlen
        self.sync = sync
        self.threshold = threshold

        ##################################################
        # Blocks
        ##################################################
        self.satellites_fixedlen_tagger_0_0_0 = satellites.fixedlen_tagger('syncword', 'packet_len', packlen*8, numpy.byte)
        self.digital_correlate_access_code_tag_bb_0_0_0 = digital.correlate_access_code_tag_bb(sync, threshold, 'syncword')
        self.blocks_unpacked_to_packed_xx_0 = blocks.unpacked_to_packed_bb(1, gr.GR_MSB_FIRST)
        self.blocks_tagged_stream_to_pdu_0_0_0 = blocks.tagged_stream_to_pdu(blocks.byte_t, 'packet_len')
        self.blocks_tagged_stream_multiply_length_0 = blocks.tagged_stream_multiply_length(gr.sizeof_char*1, 'packet_len', 1/8.0)



        ##################################################
        # Connections
        ##################################################
        self.msg_connect((self.blocks_tagged_stream_to_pdu_0_0_0, 'pdus'), (self, 'out'))
        self.connect((self.blocks_tagged_stream_multiply_length_0, 0), (self.blocks_tagged_stream_to_pdu_0_0_0, 0))
        self.connect((self.blocks_unpacked_to_packed_xx_0, 0), (self.blocks_tagged_stream_multiply_length_0, 0))
        self.connect((self.digital_correlate_access_code_tag_bb_0_0_0, 0), (self.satellites_fixedlen_tagger_0_0_0, 0))
        self.connect((self, 0), (self.digital_correlate_access_code_tag_bb_0_0_0, 0))
        self.connect((self.satellites_fixedlen_tagger_0_0_0, 0), (self.blocks_unpacked_to_packed_xx_0, 0))

    def get_packlen(self):
        return self.packlen

    def set_packlen(self, packlen):
        self.packlen = packlen

    def get_sync(self):
        return self.sync

    def set_sync(self, sync):
        self.sync = sync
        self.digital_correlate_access_code_tag_bb_0_0_0.set_access_code(self.sync)

    def get_threshold(self):
        return self.threshold

    def set_threshold(self, threshold):
        self.threshold = threshold
        self.digital_correlate_access_code_tag_bb_0_0_0.set_threshold(self.threshold)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2017 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# TODO improve this computation. probably bit reversal is not the best way to do it

def crc(data):
    poly = 0x8408
    crc = 0xffff
    for byte in data:
        # reverse byte
        byte = (byte & 0x55) <<  1 | (byte & 0xAA) >>  1
        byte = (byte & 0x33) <<  2 | (byte & 0xCC) >>  2
        byte = (byte & 0x0F) <<  4 | (byte & 0xF0) >>  4
        crc ^= byte
        for _ in range(8):
            if crc & 1: crc = (crc >> 1) ^ poly
            else: crc = crc >> 1
    crc = (crc & 0x5555) <<  1 | (crc & 0xAAAA) >>  1
    crc = (crc & 0x3333) <<  2 | (crc & 0xCCCC) >>  2
    crc = (crc & 0x0F0F) <<  4 | (crc & 0xF0F0) >>  4
    crc = (crc & 0x00FF) <<  8 | (crc & 0xFF00) >>  8
    return crc



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2018 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
# 

import numpy
from gnuradio import gr
import pmt

import pprint

class pwsat2_telemetry_parser(gr.basic_block):
    """
    docstring for block pwsat2_telemetry_parser
    """
    def __init__(self, pwsat2_path):
        gr.basic_block.__init__(self,
            name="pwsat2_telemetry_parser",
            in_sig=[],
            out_sig=[])

        import sys
        import pprint

        sys.path.append(pwsat2_path)
        sys.path.append(pwsat2_path + '/PWSat2OBC/integration_tests/')

        self.pwsat2_decoder = __import__('payload_decoder')

        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print("[ERROR] Received invalid message type. Expected u8vector")
            return
        packet = bytes(pmt.u8vector_elements(msg))

        try:
            data = self.pwsat2_decoder.PayloadDecoder.decode(packet[16:])
        except Exception as e:
            print("Could not decode telemetry beacon")
            print(e)
            return
        pprint.pprint(data)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2017 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from construct import *
from ..adapters import LinearAdapter, AffineAdapter
from .ax25 import Header16

# Beacon of type 1
General = Struct(
    'obd_mode' / Int8ul,
    'obd_active_task' / Int8ul,
    'obd_equipment_status' / Int32ul,
    'obd_cpu_error' / Int32ul,
    'obd_can_timeout_error' / Int32ul,
    'obd_wd_reset_count' / Int8ul,
    'obd_rs422m_err_count' / Int16ul,
    'obd_rs422r_err_count' / Int16ul,
    'obd_error_count' / Int16ul,
    'obd_tc_error' / Int32ul[2],
    'obd_rs422_status' / Int32ul,
    'obd_rs422_error' / Int32ul,
    'obd_rs485_status' / Int32ul,
    'obd_rs485_error' / Int32ul,
    'obd_status' / Int32ul,
    'acs_state' / Int8ul,
    'acs_omega' / Float32b[3],
    'pm_current_bp' / Int16sl[6],
    'pm_voltage_mb' / Int16ul,
    'pm_safe_operating_mode' / Int8ul,
    'pm_error_1' / Int32ul,
    'tt_tx_status_main' / Int8ul,
    'ttm_error' / Int16ul,
    'platform_fdir_ttm' / Int32ul,
    'tt_tx_status_redundant' / Int8ul,
    'ttr_error_ttr' / Int16ul,
    'platform_fdir_ttr' / Int32ul,
    'ss_error' / Int32ul[2],
    'ese_error' / Int16ul,
    'mwr_error' / Int16ul,
    'mwm_status' / Int32ul,
    'mm_error' / Int16ul[2],
    'mt_error' / Int16ul[2],
    'tt_tx_status' / Int8ul,
    'tt_error' / Int16ul)    

Temperature = LinearAdapter(10, Int16ul)

# Beacon of type 2
PowerSystem = Struct(
    'pm_voltage_sp' / Int16ul[6],
    'pm_shunt_section' / Int16ul[6],
    'pm_temp_sp' / Temperature[3],
    'pm_current_bp' / Int16ul[6],
    'pm_temp_pb' / Temperature[6],
    'pm_voltage_mb' / Int16ul,
    'pm_safe_operating_mode' / Int8ul,
    'pm_pdu_control' / Int8ul,
    'pm_temp' / Temperature[2],
    'pm_obdh_main_current' / Int16ul,
    'pm_rx_main_current' / Int16ul,
    'pm_tx_main_current' / Int16ul,
    'pm_ss_main_current' / Int16ul,
    'pm_mm_main_current' / Int16ul,
    'pm_mw_main_current' / Int16ul,
    'pm_mt_main_current' / Int16ul,
    'pm_mps_current' / Int16ul,
    'pm_tritel_current' / Int16ul,
    'pm_hstx_current' / Int16ul,
    'pm_gps_current' / Int16ul,
    'pm_mps_valve_m_current' / Int16ul,
    'pm_dom_1_current' / Int16ul,
    'pm_obdh_red_current' / Int16ul,
    'pm_rx_red_current' / Int16ul,
    'pm_tx_red_current' / Int16ul,
    'pm_ss_red_current' / Int16ul,
    'pm_mm_red_current' / Int16ul,
    'pm_mw_red_current' / Int16ul,
    'pm_mt_red_current' / Int16ul,
    'pm_es_current' / Int16ul,
    'pm_ucam' / Int16ul,
    'pm_amsat_current' / Int16ul,
    'pm_lmp_current' / Int16ul,
    'pm_eq_pl_status' / Int32ul,
    'pm_error' / Int32ul[2])

# Beacon of type 3
OBDH = Struct(
    'obd_mode' / Int8ul,
    'obd_old_mode' / Int8ul,
    'obd_active_task' / Int8ul,
    'obd_equipment_status' / Int32ul,
    'obd_equipment_health' / Int32ul,
    'obd_cpu_error' / Int32ul,
    'obd_can_status' / Int32ul,
    'obd_plcan_m_error' / Int32ul,
    'obd_plcan_r_error' / Int32ul,
    'obd_pycan_m_error' / Int32ul,
    'obd_pycan_r_error' / Int32ul,
    'obd_can_timeout_error' / Int32ul,
    'obd_hk_status' / Int32ul,
    'obd_power_time' / Int64ul,
    'obd_mode_transition' / Int64ul,
    'obd_wd_reset_count' / Int8ul,
    'obd_temp_pdu1' / Temperature,
    'obd_temp_bat1' / Temperature,
    'obd_temp_pmb' / Temperature,
    'obd_temp_hpa2' / Temperature,
    'obd_temp_hpa1' / Temperature,
    'obd_temp_tnk' / Temperature,
    'obd_temp_bat2' / Temperature,
    'obd_temp_mwm' / Temperature,
    'obd_temp_mwr' / Temperature,
    'obd_temp_mmm' / Temperature,
    'obd_temp_mmr' / Temperature,
    'obd_rs422m_err_count' / Int16ul,
    'obd_rs422r_err_count' / Int16ul,
    'obd_hk_error' / Int32ul,
    'obd_rs422_status' / Int32ul,
    'obd_rs422_error' / Int32ul,
    'obd_rs485_status' / Int32ul,
    'obd_rs485_error' / Int32ul,
    'obd_status' / Int32ul,
    'obd_error' / Int32ul,
    'obd_temp_error' / Int16ul,
    'obd_error2' / Int32ul,
    'obd_temp_error2' / Int16ul)

# Beacon of type 4
AOCS = Struct(
    'acs_state' / Int8ul,
    'acs_sun_mode' / Int8ul,
    'acs_err' / Int32ul,
    'acs_attitude_q' / Float32b[4],
    'acs_omega' / Float32b[3],
    'acs_orbit_xyz' / Float32b[3],
    'acs_orbit_v' / Float32b[3],
    'acs_state_transition' / Int64ul,
    'acs_fdir_mps_time_err' / Float32b,
    'pm_spin_rate' / Int32ul,
    'ssm_uc_pcb_temp' / Temperature,
    'ss_adc_pcb_temp' / Temperature[2],
    'ssm_topcase_temp' / Temperature,
    'ssm_sidecase_temp' / Temperature,
    'ssr_uc_pcb_temp' / Temperature,
    'ss_adc_pcb_temp' / Temperature[2],
    'ss_dcdc_temp' / Temperature,
    'ss_sidecase_temp' / Temperature,
    'ese_uc_pcb_temp' / Temperature,
    'ese_tau_temp' / Temperature,
    'mwr_temp' / Temperature,
    'mwr_if_temp' / Temperature,
    'mwm_temp' / Temperature[3],
    'mps_hpt' / Int16ul,
    'mps_lpt' / Int16ul,
    'mps_pvt_temp' / Temperature,
    'mm_dcdc_temp' / Temperature[2],
    'mt_temp' / Temperature[2])

# Beacon of type 5
FDIR = Struct(
    'obd_plcan_m_txerr_count' / Int16ul,
    'obd_plcan_m_rxerr_count' / Int16ul,
    'obd_plcan_r_txerr_count' / Int16ul,
    'obd_plcan_r_rxerr_count' / Int16ul,
    'obd_pycan_m_txerr_count' / Int16ul,
    'obd_pycan_m_rxerr_count' / Int16ul,
    'obd_pycan_r_txerr_count' / Int16ul,
    'obd_pycan_r_rxerr_count' / Int16ul,
    'obd_edac_error_count' / Int32ul,
    'obd_rs422m_err_count' / Int16ul,
    'obd_rs422r_err_count' / Int16ul,
    'obd_error_count' / Int16ul,
    'obd_hk_error' / Int32ul,
    'obd_tc_error' / Int32ul[2],
    'obd_rs422_status' / Int32ul,
    'obd_rs422_error' / Int32ul,
    'obd_rs485_status' / Int32ul,
    'obd_rs485_error' / Int32ul,
    'obd_error' / Int32ul,
    'obd_temp_error' / Int16ul,
    'acs_err' / Int32ul,
    'acs_fdir_mps_time_err' / Float32b,
    'pm_voltage_mb' / Int16ul,
    'pm_safe_operating_mode' / Int8ul,
    'pm_eq_pl_status' / Int32ul,
    'pm_undervoltage_status' / Int32ul,
    'tt_m_tx_status' / Int8ul,
    'tt_m_tx_status_1' / Int8ul,
    'tt_m_rx_status' / Int8ul,
    'tt_m_rx_status_1' / Int8ul,
    'tt_m_rx_rssi' / Int8sl,
    'tt_m_error' / Int16ul,
    'tt_m_temp' / Temperature[2],
    'tt_m_rx_afc' / LinearAdapter(1/16.0, Int8sl),
    'platform_m_fdir' / Int32ul,
    'tt_r_tx_status' / Int8ul,
    'tt_r_tx_status_1' / Int8ul,
    'tt_r_rx_status' / Int8ul,
    'tt_r_rx_status_1' / Int8ul,
    'tt_r_rx_rssi' / Int8sl,
    'tt_r_error' / Int16ul,
    'tt_r_temp' / Temperature[2],
    'tt_r_rx_afc' / LinearAdapter(1/16.0, Int8sl),
    'platform_r_fdir' / Int32ul[3])

TriTmp = AffineAdapter(0.5, -40, Int8ul)

# Beacon of type 6
Payload = Struct(
    'tri_tmp_xyz' / TriTmp[3],
    'tri_tmp_psu' / TriTmp,
    'tri_tmp_cpu' / TriTmp,
    'tri_tmp_adc' / TriTmp[3],
    'tri_uinput' / LinearAdapter(150, Int8ul),
    'tri_iinput' / LinearAdapter(2, Int8ul),
    'tri_60V' / LinearAdapter(300, Int8ul),
    'tri_5V' / LinearAdapter(30, Int8ul),
    'tri_3_3V' / LinearAdapter(20, Int8ul),
    'tri_neg10V' / LinearAdapter(100, Int8ul),
    'tri_6_5V' / LinearAdapter(50, Int8ul),
    'tri_neg6_5V' / LinearAdapter(50, Int8ul),
    'tri_mode' / Int8ul,
    'tri_freq' / Int8ul,
    'tri_error' / Int8ul,
    'eeprom' / Int8ul,
    'lmp_tt_psu' / Int8sl,
    'lmp_vt_p12' / LinearAdapter(1/0.078, Int8ul),
    'lmp_vt_m12' / LinearAdapter(-1/0.078, Int8ul),
    'lmp_vt_p5' / LinearAdapter(1/0.029, Int8ul),
    'lmp_vt_m5' / LinearAdapter(-1/0.029, Int8ul),
    'lmp_ct_dig' / LinearAdapter(1/1.259, Int8ul),
    'lmp_vt_dig' / LinearAdapter(1/0.02, Int8ul),
    'lmp_mem' / LinearAdapter(1/4096.0, Int8ul),
    'lmp_ofs' / LinearAdapter(1/4.88, Int8ul),
    'lmp_sw' / Int24ul,
    'pcam_mcur_curr' / Int8ul,
    'pcam_img_curr' / Int8ul,
    'pcam_mcu_temp' / Temperature,
    'pcam_img_temp' / Temperature,
    'pcam_dcdc_temp' / Temperature,
    'scam_mcur_curr' / Int8ul,
    'scam_img_curr' / Int8ul,
    'scam_ram_curr' / Int8ul,
    'scam_mcu_temp' / Temperature,
    'scam_img_temp' / Temperature,
    'scam_sdr_temp' / Temperature[2],
    'ams_obc_p_up' / Int32ul,
    'ams_obc_p_up_dropped' / Int32ul,
    'ams_obc_mem_stat_ram' / Int32ul,
    'ams_obc_mem_stat_flash' / Int32ul,
    'ams_eps_dcdc_t' / Int8ul,
    'ams_vhf_fm_pa_t' / Int8ul,
    'ams_vhf_bpsk_pa_t' / Int8ul,
    'stx_vol' / LinearAdapter(50, Int8ul)[2],
    'stx_cur' / LinearAdapter(50, Int8ul)[2],
    'stx_temp' / AffineAdapter(0.5, 230, Int8ul)[4],
    'stx_stat' / Int32ul,
    'stx_com' / Int32ul,
    'stx_mem' / Int32ul,
    'gps_current_3V3' / Int16ul,
    'gps_current_5V' / Int16ul,
    'gps_week' / Int16ul,
    'gps_temperature' / Temperature[2],
    'gps_frend_m_volt' / Int16ul,
    'gps_frend_r_volt' / Int16ul,
    'gps_seconds_of_week' / Int32ul,
    'ade_in_estimator_on' / Int8ul,
    'ade_in_omega' / Int8ul,
    'ade_oprq_q' / Float32b[3])

eseo = Struct(
    'header' / Header16,
    'type' / Int8ul,
    'byte' / Int8ul,
    'data' / Switch(this.type, {3 : General, 4 : PowerSystem, 5 : OBDH, 6 : AOCS, 7 : FDIR, 8 : Payload})
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
# 

from construct import *
from .csp import CSPHeader
from ..adapters import UNIXTimestampAdapter, LinearAdapter

Voltage = LinearAdapter(1000.0, Int16ub)
Timestamp = UNIXTimestampAdapter(Int32ub)

EPS = Struct(
    'timestamp' / Timestamp,
    'vboost' / Voltage[3],
    'vbatt' / Voltage,
    'curout' / Int16ub[7],
    'curin' / Int16ub[3],
    'cursun' / Int16ub,
    'cursys' / Int16ub,
    'eps_temp' / Int16sb[6],
    'battmode' / Int8ub
    )

Temperature = LinearAdapter(10.0, Int16sb)
COM = Struct(
    'timestamp' / Timestamp,
    'temp_brd' / Temperature,
    'temp_pa' / Temperature,
    'last_rssi' / Int16sb,
    'last_rferr' / Int16sb,
    'bgnd_rssi' / Int16sb
    )

OBC = Struct(
    'timestamp' / Timestamp,
    'cur_gssb' / Int16ub[2],
    'cur_flash' / Int16ub,
    'temp' / Temperature[2]
    )

ADCS = Struct(
    'timestamp' / Timestamp,
    'cur_gssb' / Int16ub[2],
    'cur_flash' / Int16ub,
    'cur_pwm' / Int16ub,
    'cur_gps' / Int16ub,
    'cur_wde' / Int16ub,
    'temp' / Temperature[2]
    )

ADSB = Struct(
    'timestamp' / Timestamp,
    'cur5v0brd' / Int16ub,
    'cur3v3brd' / Int16ub,
    'cur3v3sd' / Int16ub,
    'cur1v2' / Int16ub,
    'cur2v5' / Int16ub,
    'cur3v3fpga' / Int16ub,
    'cur3v3adc' / Int16ub,
    'last_icao' / Hex(Int32ub),
    'last_lat' / Float32b,
    'last_lon' / Float32b,
    'last_alt' / Int32ub,
    'last_time' / Timestamp
    )

gomx_3 = Struct(
    'csp_header' / CSPHeader,
    'beacon_type' / Int8ub,
    'beacon' / If((this.csp_header.destination == 10) & (this.csp_header.destination_port == 30) & (this.csp_header.source == 1) & (this.beacon_type == 0),
                Struct(
                'eps' / EPS,
                'com' / COM,
                'obc' / OBC,
                'adcs' / ADCS,
                'adsb' / ADSB
                )
            )
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, digital
from ... import nrzi_decode, hdlc_deframer

class ua01_deframer(gr.hier_block2):
    """
    Hierarchical block to deframe UA01 non-conformant AX.25.

    The input is a float stream of soft symbols. The output are PDUs
    with UA01 frames.

    The input should be doubly NRZ-I encoded and G3RUH scrambled.

    Args:
        options: Options from argparse
    """
    def __init__(self, options = None):
        gr.hier_block2.__init__(self, "ua01_deframer",
            gr.io_signature(1, 1, gr.sizeof_float),
            gr.io_signature(0, 0, 0))
        self.message_port_register_hier_out('out')

        self.slicer = digital.binary_slicer_fb()
        self.nrzi0 = nrzi_decode()
        self.nrzi1 = nrzi_decode()
        self.descrambler = digital.descrambler_bb(0x21, 0, 16)
        self.deframer = hdlc_deframer(True, 10000)

        self.connect(self, self.slicer, self.nrzi0, self.nrzi1,
                         self.descrambler, self.deframer)
        self.msg_connect((self.deframer, 'out'), (self, 'out'))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2017,2020 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import numpy
from gnuradio import gr
import collections
import pmt
import datetime
import struct
import warnings

from .kiss import *
from .submit import parse_time

class pdu_to_kiss(gr.basic_block):
    """
    docstring for block pdu_to_kiss
    """
    def __init__(self, control_byte = True, include_timestamp = False,
                     initial_timestamp = ''):
        gr.basic_block.__init__(self,
            name="pdu_to_kiss",
            in_sig=None,
            out_sig=None)
        self.control_byte = control_byte
        self.include_timestamp = include_timestamp
        self.initial_timestamp = parse_time(initial_timestamp) \
            if initial_timestamp != '' else None
        self.start_timestamp = datetime.datetime.utcnow()

        if not control_byte and include_timestamp:
            warnings.warn('Using no control byte and timestamps in pdu_to_kiss will usually give problems')

        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('out'))

    def create_timestamp(self):
        """Returns the timestamp as a KISS control frame with control byte 09

        The timestamp is the number of milliseconds elapsed since the UNIX
        epoch according to UTC and not counting leap seconds, stored as a
        big-endian 64 bit unsigned integer
        """
        # it's tempting to use time.time() to get the timestamp
        # but the docs say that its epoch is often 1970-01-01,
        # but they don't promise that it's always that
        # we use datetime.datetime() instead
        epoch = datetime.datetime(1970, 1, 1)
        now = datetime.datetime.utcnow()
        if self.initial_timestamp:
            now = now - self.start_timestamp + self.initial_timestamp
        timestamp = (now - epoch).total_seconds()
        timestamp_int = round(timestamp * 1e3)
        control = b'\x09'
        return control + struct.pack('>Q', timestamp_int)

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print("[ERROR] Received invalid message type. Expected u8vector")
            return

        control = [numpy.uint8(0)] if self.control_byte else []
        frame = [FEND] + control + kiss_escape(pmt.u8vector_elements(msg)) + [FEND]

        if self.include_timestamp:
            timestamp_frame = [FEND] + kiss_escape(self.create_timestamp()) + [FEND]
            self.message_port_pub(pmt.intern('out'), pmt.cons(pmt.PMT_NIL, pmt.init_u8vector(len(timestamp_frame), timestamp_frame)))
            
        self.message_port_pub(pmt.intern('out'), pmt.cons(pmt.PMT_NIL, pmt.init_u8vector(len(frame), frame)))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

'''
gr-satellites filereceiver

This module contains the File Receiver class and child classes
This class is used to reassemble files transmitted in chunks
'''

from .by70_1 import by70_1
from .dsat import dsat
from .k2sat import k2sat
from .lucky7 import lucky7
from .sat_1kuns_pf import sat_1kuns_pf
from .smogp import smogp
from .swiatowid import swiatowid



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# Based on the decoder by Xerbo
# https://github.com/Xerbo/Lucky7-Decoder

import struct

from .imagereceiver import ImageReceiver

class ImageReceiverLucky7(ImageReceiver):
    def chunk_sequence(self, chunk):
        return struct.unpack('>H', chunk[3:5])[0]

    def chunk_size(self):
        return 28
    
    def chunk_data(self, chunk):
        return chunk[7:]

    def file_size(self, chunk):
        return self.chunk_size() * struct.unpack('>H', chunk[5:7])[0]

    def parse_chunk(self, chunk):
        address = struct.unpack('>H', chunk[1:3])[0]
        if address < 0xC000 \
          or address >= 0xC000 + self.file_size(chunk)/self.chunk_size():
            return None
        return chunk    

lucky7 = ImageReceiverLucky7



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, blocks, digital
from ... import decode_rs, ngham_packet_crop, ngham_remove_padding, ngham_check_crc
from ...hier.sync_to_pdu_packed import sync_to_pdu_packed
from ...hier.ccsds_descrambler import ccsds_descrambler
from ...utils.options_block import options_block

_syncword = '01011101111001100010101001111110'

class ngham_deframer(gr.hier_block2, options_block):
    """
    Hierarchical block to deframe NGHam frames

    These use a CCSDS scrambler and CCSDS Reed-Solomon
    Reed-Solomon decoding is currently not implemented

    The input is a float stream of soft symbols. The output are PDUs
    with frames.

    Args:
        decode_rs: use Reed-Solomon decoding (bool)
        syncword_threshold: number of bit errors allowed in syncword (int)
        options: Options from argparse
    """
    def __init__(self, decode_rs = False, syncword_threshold = None, options = None):
        gr.hier_block2.__init__(self, "ccsds_rs_deframer",
            gr.io_signature(1, 1, gr.sizeof_float),
            gr.io_signature(0, 0, 0))
        options_block.__init__(self, options)
        
        self.message_port_register_hier_out('out')

        if decode_rs:
            raise ValueError('NGHam Reed-Solomon decoding not implemented yet')
        
        if syncword_threshold is None:
            syncword_threshold = self.options.syncword_threshold

        self.slicer = digital.binary_slicer_fb()
        self.deframer = sync_to_pdu_packed(packlen = 255 + 3,\
                                           sync = _syncword,\
                                           threshold = syncword_threshold)
        self.crop = ngham_packet_crop()
        self.pdu2tag = blocks.pdu_to_tagged_stream(blocks.byte_t, 'packet_len')
        self.unpack = blocks.packed_to_unpacked_bb(1, gr.GR_MSB_FIRST)
        self.taglength = blocks.tagged_stream_multiply_length(gr.sizeof_char*1, 'packet_len', 8)
        self.tag2pdu = blocks.tagged_stream_to_pdu(blocks.byte_t, 'packet_len')
        self.scrambler = ccsds_descrambler()
        self.padding = ngham_remove_padding()
        self.crc = ngham_check_crc(self.options.verbose_crc)

        self.connect(self, self.slicer, self.deframer)
        self.msg_connect((self.deframer, 'out'), (self.crop, 'in'))
        self.msg_connect((self.crop, 'rs16'), (self.pdu2tag, 'pdus'))
        self.msg_connect((self.crop, 'rs32'), (self.pdu2tag, 'pdus'))
        self.connect(self.pdu2tag, self.unpack, self.taglength, self.tag2pdu)
        self.msg_connect((self.tag2pdu, 'pdus'), (self.scrambler, 'in'))
        self.msg_connect((self.scrambler, 'out'), (self.padding, 'in'))
        self.msg_connect((self.padding, 'out'), (self.crc, 'in'))
        self.msg_connect((self.crc, 'ok'), (self, 'out'))

    _default_sync_threshold = 4
        
    @classmethod
    def add_options(cls, parser):
        """
        Adds NGHam deframer specific options to the argparse parser
        """
        parser.add_argument('--syncword_threshold', type = int, default = cls._default_sync_threshold, help = 'Syncword bit errors [default=%(default)r]')
#        parser.add_argument('--verbose_rs', action = 'store_true', help = 'Verbose RS decoder')
        parser.add_argument('--verbose_crc', action = 'store_true', help = 'Verbose CRC check')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
# 

from construct import *

from ..ccsds.telemetry import PrimaryHeader, OCFTrailer

trisat = Struct(
    'tm_primary_header' / PrimaryHeader,
    'payload' / Bytes(211),
    'ocf' / OCFTrailer,
    'crc' / Int16ub
    )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2016 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import numpy
from gnuradio import gr
import pmt

class ks1q_header_remover(gr.basic_block):
    """
    docstring for block ks1q_header_remover
    """
    def __init__(self, verbose):
        gr.basic_block.__init__(self,
            name="ks1q_header_remover",
            in_sig=[],
            out_sig=[])
        self.verbose = verbose
        self.message_port_register_in(pmt.intern('in'))
        self.set_msg_handler(pmt.intern('in'), self.handle_msg)
        self.message_port_register_out(pmt.intern('out'))

    def handle_msg(self, msg_pmt):
        msg = pmt.cdr(msg_pmt)
        if not pmt.is_u8vector(msg):
            print("[ERROR] Received invalid message type. Expected u8vector")
            return
        packet = bytes(pmt.u8vector_elements(msg))

        if len(packet) <= 3:
            return

        if self.verbose:
            print('Spacecraft ID', packet[:2].hex())
            if packet[2] == 0x50:
                print('CSP downlink, protocol version 0')
            else:
                print('Unknown packet type')

        data = packet[3:]
        self.message_port_pub(pmt.intern('out'),
                                  pmt.cons(pmt.PMT_NIL,
                                           pmt.init_u8vector(len(data), data)))





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2018-2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
# 

from construct import *
from ..adapters import *
from .ax25 import Header

Timestamp = UNIXTimestampAdapter(Int32ul)

Callsign = Struct(
    'callsign' /  Bytes(5))

OBC = Struct(
    'obc_mode' / Int8ul,
    'obc_reset_counter' / Int32ul,
    'obc_uptime' / Int32ul)

Gyro = Struct(
    'gyro_norm' / Int8ul)

EPS = Struct(
    'eps_counter_boot' / Int32ul,
    'eps_last_boot_cause' / Int8ul,
    'eps_battery_mode' / Int8ul)

Timestamp_Struct = Struct(
    'timestamp' / Timestamp)

OBC_Temp = Struct(
    'obc_temp' / AffineAdapter(1, 128, Int8ul),
    'obc_daughter_board_temp' / AffineAdapter(1, 128, Int8ul))

EPS_Temp = Struct(
    'eps_battery_temp' / AffineAdapter(1, 128, Int8ul),
    'eps_board_temp' / AffineAdapter(1, 128, Int8ul))

Ants = Struct(
    'ants_temp' / AffineAdapter(1, 128, Int8ul))

TRXVU_Temp = Struct(
    'trxvu_temp' / AffineAdapter(1, 128, Int8ul))

ADCS = Struct(
    'adcs_temp' / AffineAdapter(1, 128, Int8ul))

OBC_Voltages = Struct(
'obc_3v3_voltage' / LinearAdapter(10.0, Int8ul),
'obc_5v0_voltage' / LinearAdapter(10.0, Int8ul))

TRXVU_Voltage = Struct(
'trxvu_voltage' / LinearAdapter(10.0, Int8ul))

EPS_Batt_Voltage = Struct(
'eps_batt_voltage' / LinearAdapter(10.0, Int8ul))

OBC_Current = Struct(
'obc_5.0_current' / LinearAdapter(1000.0, Int16ul))

EPS_Currents = Struct(
'eps_total_pv_current' / LinearAdapter(1000.0, Int16ul),
'eps_total_system_current' / LinearAdapter(1000.0, Int16ul))

mysat1 = Struct(
    'header' / Header,
    'callsign' / Callsign,
    'obc' / OBC,
    'gyro' / Gyro,
    'eps' / EPS,
    'timestamp' / Timestamp_Struct,
    'obc_temp' / OBC_Temp,
    'eps_temp' / EPS_Temp,
    'ants' / Ants,
    'trxvu_temp' / TRXVU_Temp,
    'adcs' / ADCS,
    'obc_voltages' / OBC_Voltages,
    'trxvu_voltage' / TRXVU_Voltage,
    'eps_batt_voltage' / EPS_Batt_Voltage,
    'obc_current' / OBC_Current,
    'eps_currents' / EPS_Currents)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, digital
from ... import check_ao40_uncoded_crc
from ...hier.sync_to_pdu_packed import sync_to_pdu_packed
from ...utils.options_block import options_block

_syncword = '00111001000101011110110100110000'

class ao40_uncoded_deframer(gr.hier_block2, options_block):
    """
    Hierarchical block to deframe the AO-40 uncoded framing.

    The input is a float stream of soft symbols. The output are PDUs
    with frames.

    Args:
        syncword_threshold: number of bit errors allowed in syncword (int)
        options: Options from argparse
    """
    def __init__(self, syncword_threshold = None, options = None):
        gr.hier_block2.__init__(self, "tt64_deframer",
            gr.io_signature(1, 1, gr.sizeof_float),
            gr.io_signature(0, 0, 0))
        options_block.__init__(self, options)
        
        self.message_port_register_hier_out('out')

        if syncword_threshold is None:
            syncword_threshold = self.options.syncword_threshold

        self.slicer = digital.binary_slicer_fb()
        self.deframer = sync_to_pdu_packed(packlen = 512 + 2,\
                                    sync = _syncword,\
                                    threshold = syncword_threshold)
        self.crc = check_ao40_uncoded_crc(self.options.verbose_crc)
        
        self.connect(self, self.slicer, self.deframer)
        self.msg_connect((self.deframer, 'out'), (self.crc, 'in'))
        self.msg_connect((self.crc, 'ok'), (self, 'out'))

    _default_sync_threshold = 3
        
    @classmethod
    def add_options(cls, parser):
        """
        Adds Astrocast 9k6 deframer specific options to the argparse parser
        """
        parser.add_argument('--syncword_threshold', type = int, default = cls._default_sync_threshold, help = 'Syncword bit errors [default=%(default)r]')
        parser.add_argument('--verbose_crc', action = 'store_true', help = 'Verbose CRC decoder')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2017,2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
# 

import datetime

from construct import *
from ..adapters import UNIXTimestampAdapter
from .csp import CSPHeader

Timestamp = UNIXTimestampAdapter(Int32sb)

class BatteryAdapter(Adapter):
    def _encode(self, obj, context, path = None):
        return int((obj - 4420) / 16.0)
    def _decode(self, obj, context, path = None):
        return obj * 16 + 4420
Battery = BatteryAdapter(Int8ub)

class CurrInAdapter(Adapter):
    def _encode(self, obj, context, path = None):
        return int(obj * 255.0 / 2700.0)
    def _decode(self, obj, context, path = None):
        return obj * 2700.0 / 255.0
CurrIn = CurrInAdapter(Int8ub)

class CurrOutAdapter(Adapter):
    def _encode(self, obj, context, path = None):
        return int(obj * 255.0 / 4000.0)
    def _decode(self, obj, context, path = None):
        return obj * 4000.0 / 255.0
CurrOut = CurrOutAdapter(Int8ub)

class Curr3Adapter(Adapter):
    def _encode(self, obj, context, path = None):
        return int(obj * 255.0 / 5500.0)
    def _decode(self, obj, context, path = None):
        return obj * 5500.0 / 255.0
Curr3 = Curr3Adapter(Int8ub)

class Curr5Adapter(Adapter):
    def _encode(self, obj, context, path = None):
        return int(obj * 255.0 / 4500.0)
    def _decode(self, obj, context, path = None):
        return obj * 4500.0 / 255.0
Curr5 = Curr5Adapter(Int8ub)
                      
au03 = Struct(
        'csp_header' / CSPHeader,
        'timestamp' / Timestamp,
        'callsign' / Bytes(6),
        'flags' / Byte,
        'batt_voltage' / Battery,
        'current_in' / CurrIn,
        'current_out' / CurrOut,
        'rail3_current' / Curr3,
        'rail5_current' / Curr5,
        'com_temp' / Int8sb,
        'eps_temp' / Int8sb,
        'bat_temp' / Int8sb
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2018 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
# 

import datetime

from construct import *
import construct.core
from ..adapters import LinearAdapter, UNIXTimestampAdapter

# Number of sensors
VOLT_SENSORS = 7
CUR_SENSORS = 6
TEMP_SENSORS = 7
IRR_SENSORS = 6

def sensor(context):
    if context._.beacon_id == 'VoltageEvolution':
        return 'voltage'
    if context._.beacon_id == 'CurrentEvolution':
        return 'current'
    if context._.beacon_id == 'TemperatureEvolution':
        return 'temperature'
    if context._.beacon_id == 'IrradianceEvolution':
        return 'irradiance'
    if context._.beacon_id == 'StateOfChargeEvolution':
        return 'state_of_charge'
    idx = context._index
    if idx < VOLT_SENSORS:
        return 'voltage'
    idx -= VOLT_SENSORS
    if idx < CUR_SENSORS:
        return 'current'
    idx -= CUR_SENSORS
    if idx < TEMP_SENSORS:
        return 'temperature'
    return 'irradiance'

VoltOrIrrAdapter = LinearAdapter(1000, Int16ub)
TempAdapter = LinearAdapter(10, Int16ub)

class DeltaTAdapter(Adapter):
#    def _encode(self, obj, context, path = None):
# TODO
    def _decode(self, obj, context, path = None):
        if obj == 0:
            raise construct.core.StreamError
        delta_t = obj
        t1 = context._.last_timestamp
        t2 = t1 - datetime.timedelta(seconds = delta_t - 1 + context._.delta_min_delay)
        if context._.beacon_id != 'CurrentState':
            context._.last_timestamp = t2
        return t2

def set_delta_min_delay(obj, context):
    context.delta_min_delay = 0 if obj == 'CurrentState' else 300
    
def set_last_timestamp(obj, context):
    context.last_timestamp = obj

def set_context(obj, context):
    set_delta_min_delay(obj, context)
    set_last_timestamp(obj, context)

BeaconID = Enum(Int8ub,\
    CurrentState = 0xB0,\
    VoltageEvolution = 0xB1,\
    CurrentEvolution = 0xB2,\
    TemperatureEvolution = 0xB3,\
    IrradianceEvolution = 0xB4,\
    StateOfChargeEvolution = 0xB5)

Timestamp = UNIXTimestampAdapter(Int32sb)

Value = Switch(this.parameter, {'voltage' : VoltOrIrrAdapter, 'irradiance' : VoltOrIrrAdapter, 'temperature' : TempAdapter}, default = Int16ub)

LongData = Struct(\
    Const(b'\x00'),\
    'timestamp' / DeltaTAdapter(Int24ub),\
    'parameter' / Computed(sensor),\
    'value' / Value)

ShortData = Struct(\
    'timestamp' / DeltaTAdapter(Int8ub),\
    'parameter' / Computed(sensor),\
    'value' / Value)

Data = Select(LongData, ShortData)

sat_3cat_1 = Struct(\
    Const(b'\x00'),\
    'beacon_id' / BeaconID * set_delta_min_delay,\
    'sc_time' / Timestamp * set_last_timestamp,\
    'state_of_charge' / Int8ub,\
    'sensor_id' / Int8ub,\
    'data' / GreedyRange(Data))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Daniel Estevez <daniel@destevez.net>
#
# This file is part of gr-satellites
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, digital
from ... import nrzi_decode, reflect_bytes, decode_rs, check_astrocast_crc
from ...hier.sync_to_pdu_packed import sync_to_pdu_packed
from ...utils.options_block import options_block

_syncword = '0111010111111010110000011010001101011000110100000110010001110110'

class astrocast_fx25_deframer(gr.hier_block2, options_block):
    """
    Hierarchical block to deframe the Astrocast (somewhat non-compliant) FX.25

    The input is a float stream of soft symbols. The output are PDUs
    with frames.

    Args:
        syncword_threshold: number of bit errors allowed in syncword (int)
        nrzi: use NRZ-I instead of NRZ (bool)
        options: Options from argparse
    """
    def __init__(self, syncword_threshold = None, nrzi = True, options = None):
        gr.hier_block2.__init__(self, "astrocast_fx25_deframer",
            gr.io_signature(1, 1, gr.sizeof_float),
            gr.io_signature(0, 0, 0))
        options_block.__init__(self, options)
        
        self.message_port_register_hier_out('out')

        if syncword_threshold is None:
            syncword_threshold = self.options.syncword_threshold

        self.slicer = digital.binary_slicer_fb()
        if nrzi:
            self.nrzi = nrzi_decode()
        self.deframer = sync_to_pdu_packed(packlen = 255,\
                                           sync = _syncword,\
                                           threshold = syncword_threshold)
        self.reflect = reflect_bytes()
        self.rs = decode_rs(True, 1)
        self.crc = check_astrocast_crc(self.options.verbose_crc)

        blocks = [self, self.slicer]
        if nrzi:
            blocks.append(self.nrzi)
        blocks.append(self.deframer)
        
        self.connect(*blocks)
        self.msg_connect((self.deframer, 'out'), (self.reflect, 'in'))
        self.msg_connect((self.reflect, 'out'), (self.rs, 'in'))
        self.msg_connect((self.rs, 'out'), (self.crc, 'in'))
        self.msg_connect((self.crc, 'ok'), (self, 'out'))

    _default_sync_threshold = 8
        
    @classmethod
    def add_options(cls, parser):
        """
        Adds Astrocast FX.25 deframer specific options to the argparse parser
        """
        parser.add_argument('--syncword_threshold', type = int, default = cls._default_sync_threshold, help = 'Syncword bit errors [default=%(default)r]')
        parser.add_argument('--verbose_rs', action = 'store_true', help = 'Verbose RS decoder')
        parser.add_argument('--verbose_crc', action = 'store_true', help = 'Verbose CRC decoder')
